/***************************************************************************
                          HermodCall.cpp  -  description
                             -------------------
    begin                : Fri Feb 21 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/

#include "HermodCall.h"
#include "../MiniXml/MiniXML.h"


HermodCall::HermodCall()
{
	m_methodName = "";
	m_url = "";
	m_connectionId = "";
	m_soccketFd = -1;
}//HermodCall

HermodCall::~HermodCall()
{
#ifdef WIN32
	if (m_soccketFd > 0) closesocket(m_soccketFd);
#else
	if (m_soccketFd > 0) close(m_soccketFd);
#endif
}//HermodCall

HermodResponse *HermodCall::setUserPassword(char *user, char *password)
{
	vector<HermodParameter> oldParams;
	string oldMethodName;
	HermodParameter *param;
	HermodResponse *response;

	oldParams = m_params;
	m_params.clear();
	oldMethodName = m_methodName;

	m_methodName = "set-user-password";
	param = new HermodParameter("user",user);
	m_params.push_back(*param);
	param = new HermodParameter("password",password);
	m_params.push_back(*param);

	response = invoke();

	m_params.clear();
	m_params = oldParams;
	m_methodName = oldMethodName;
	return response;
}//setUserPassword

HermodResponse *HermodCall::getFreeConnection()
{
	vector<HermodParameter> oldParams;
	string oldMethodName;
	HermodResponse *response;

	oldParams = m_params;
	m_params.clear();
	oldMethodName = m_methodName;

	m_methodName = "get-free-connection";
	m_params.clear();

	response = invoke();

	m_params = oldParams;
	m_methodName = oldMethodName;
	return response;
}//getFreeConnection

HermodResponse *HermodCall::getConnection()
{
	vector<HermodParameter> oldParams;
	string oldMethodName;
	HermodResponse *response;

	oldParams = m_params;
	m_params.clear();
	oldMethodName = m_methodName;

	m_methodName = "get-connection";
	m_params.clear();

	response = invoke();

	m_params = oldParams;
	m_methodName = oldMethodName;
	return response;
}//getConnection

HermodResponse *HermodCall::releaseConnection()
{
	vector<HermodParameter> oldParams;
	string oldMethodName;
	HermodResponse *response;

	oldParams = m_params;
	m_params.clear();
	oldMethodName = m_methodName;

	m_methodName = "release-connection";
	m_params.clear();

	response = invoke();

	m_connectionId = "";
	m_params = oldParams;
	m_methodName = oldMethodName;
	return response;
}//releaseConnection

HermodResponse *HermodCall::invoke()
{
	string str, xmlRpcStr;
	HermodResponse *response;
	HermodParameter param, *pParam;
	vector<HermodParameter> params;
	MiniXML miniXml;
	Element *xmlTree, *elm, *subElm;
	char numBuff[256], *line, *attr, *cp;
	int i;

	if (m_connectionId.length() > 0) xmlRpcStr = "<methodCall connection-id='"+m_connectionId+"'>\n";
	else xmlRpcStr = "<methodCall>\n";
	xmlRpcStr += " <methodName>"+m_methodName+"</methodName>\n";
	xmlRpcStr += " <params>\n";

	for (i=0; i< m_params.size(); i++) {
		param = m_params.at(i);
		xmlRpcStr += "  <param name='"+param.m_name+"'>\n";
		xmlRpcStr += "   <string>"+param.m_value+"</string>\n";
		xmlRpcStr +="  </param>\n";
	}

	xmlRpcStr += " </params>\n";
	xmlRpcStr += "</methodCall>\n";

	if (!connectToServer()) {
		HermodFault *fault = new HermodFault();
		fault->setFaultCode(10);
		fault->setFaultString(m_errorStr);
		response = new HermodResponse();
		response->setFault(fault);
		return response;
	}

	sprintf(numBuff,"%d",xmlRpcStr.length());
	str = "GET / HTTP/1.0\r\nAccept: text/xml\r\ncontent-length: ";
	str += numBuff;
	str += "\r\n\r\n"+xmlRpcStr;
	if (sendStr(str) < 0) {
		HermodFault *fault = new HermodFault();
		fault->setFaultCode(10);
		fault->setFaultString("Error while sending to server");
		response = new HermodResponse();
		response->setFault(fault);
		return response;
	}

	if (!reciveStr(str)) {
		HermodFault *fault = new HermodFault();
		fault->setFaultCode(10);
		fault->setFaultString("Error while sending to server");
		response = new HermodResponse();
		response->setFault(fault);
		return response;
	}

#ifdef WIN32
	if (m_soccketFd > 0) closesocket(m_soccketFd);
#else
	if (m_soccketFd > 0) close(m_soccketFd);
#endif
	m_soccketFd = -1;

	cp = (char *)str.c_str();
	i = 0;
	while (*cp) {
		if (*cp=='\r' && *(cp+1)=='\n' && *(cp+2)=='\r' && *(cp+3)=='\n') {
			i += 4;
			break;
		}
		cp++;
		i++;
	}
	str = str.substr(i,str.length()-i);

	response = new HermodResponse();
	miniXml.Parse((char *)str.c_str());
	xmlTree = miniXml.GetTree();
	xmlTree->InitSearch();
	if ((elm=xmlTree->SearchFor("fault"))!=NULL) { //the server responded with a fault
		HermodFault *fault = new HermodFault();
		if ((elm=xmlTree->SearchFor("struct"))==NULL) { //<struct>
			fault->setFaultCode(10);
			fault->setFaultString("The server responded with a fault. But there are syntax errors in it.");
			response->setFault(fault);
			return response;
		}
		if ((elm=elm->GetSubElements())==NULL) return NULL;
		for (i=0; i<2; i++) { //we have only two values in a fault
			if ((subElm=elm->GetFirstSubElement())!=NULL) { //<name>
				if (strcmp(subElm->GetValue(),"faultCode")==0) {
					if ((subElm=elm->GetNextSubElement())!=NULL) { //<value>
						if ((subElm=subElm->GetFirstSubElement())!=NULL) { //<int>
							fault->setFaultCode(atol(subElm->GetValue()));
							elm = elm->GetNextElement();
							continue;
						}
					}
				}
				else if (strcmp(subElm->GetValue(),"faultString")==0) {
					if ((subElm=elm->GetNextSubElement())!=NULL) { //<value>
						if ((subElm=subElm->GetFirstSubElement())!=NULL) { //<string>
							fault->setFaultString(subElm->GetValue());
							elm = elm->GetNextElement();
							continue;
						}
					}
				}
			}
		}
		response->setFault(fault);
		return response;
	}

	if ((elm=xmlTree->SearchFor("methodResponse"))==NULL) { //<methodResponse>
		HermodFault *fault = new HermodFault();
		fault->setFaultCode(10);
		fault->setFaultString("There is a syntax error in the server response.");
		response->setFault(fault);
		return response;
	}
	if ((line=elm->GetAttrubuteValue("connection-id"))!=NULL) {
		m_connectionId = line;
		response->setConnectionId(line);
	}
	else m_connectionId = "";
        
	if ((elm=xmlTree->SearchFor("params"))==NULL) { //<params>
		HermodFault *fault = new HermodFault();
		fault->setFaultCode(10);
		fault->setFaultString("There is a syntax error in the server response.");
		response->setFault(fault);
		return response;
	}
	if ((elm=elm->GetFirstSubElement())!=NULL) {
		while (elm!=NULL) {
			if ((attr=elm->GetAttrubuteValue("name"))!=NULL) {
				pParam = new HermodParameter();
				pParam->setName(attr);
				if ((subElm = elm->GetFirstSubElement())!=NULL) { //<value>
					if ((subElm = subElm->GetFirstSubElement())!=NULL) { //<string>
						pParam->setValue(subElm->GetValue());
						params.push_back(*pParam);
					}
				}
			}
			elm = elm->GetNextElement();
		}
	}
	response->setParams(params);

	return response;
}//invoke


BOOL HermodCall::connectToServer()
{
	struct sockaddr_in servAddr;
	unsigned long inaddr;
	struct hostent *hp;
	string host;
	short port = 80;
	int i, j, k;

	if ((i=m_url.find_first_of("http://")) < 0) {
		m_errorStr = "Ilegal URL";
		return FALSE;
	}
	i += 7;
	if ((j=m_url.find_first_of(":", i)) >= 0) {
		j++;
		if ((k=m_url.find_first_of("/",j)) < 0) k = m_url.length();
		port = atol(m_url.substr(j,k-j).c_str());
	}
	else if ((j=m_url.find_first_of("/", i)) < 0) j = m_url.length();
	host = m_url.substr(i,j-i-1);

	memset(&servAddr,0,sizeof(struct sockaddr_in));
	servAddr.sin_family = AF_INET;
	servAddr.sin_port = htons(port);
	if ((inaddr=inet_addr(host.c_str())) != INADDR_NONE) { //First try dotted-decimal
		servAddr.sin_addr.s_addr = inaddr;
	}
	else { //try by name
		if ((hp=gethostbyname(host.c_str()))==NULL) {
			m_errorStr = "Can't find the host "+host;
			return FALSE;
		}
		else { //Found by name!
			memcpy((void *)&servAddr.sin_addr.s_addr, hp->h_addr, hp->h_length);
		}
	}

	if ((m_soccketFd=socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		m_errorStr = "Can't open socket";
		return FALSE;
	}
	if (connect(m_soccketFd, (struct sockaddr *) &servAddr, sizeof(struct sockaddr_in)) > 0) {
		m_errorStr = "Can't connect ot server";
		return FALSE;
	}

	return TRUE;
}//connectToServer

int HermodCall::sendStr(string str)
/*
 * This method send a string to the client socket
 */
{
	int nleft, nwritten;
	const char *ptr;

	nleft = str.length();
	ptr = str.c_str();
	while (nleft > 0) {
		nwritten = send(m_soccketFd, ptr, nleft, 0);
		if (nwritten < 0) return -1;
		nleft -= nwritten;
		ptr += nwritten;
	}
	return nleft;
}//sendStr

BOOL HermodCall::reciveStr(string &str)
{
	int nleft, nread;
	char buffer[513], *ptr;

	str = "";
	nleft = 512;
	while (nleft > 0) {
		ptr = buffer;
		nread  = recv(m_soccketFd, ptr, nleft, 0);
		if (nread < 0) return FALSE;
		if (nread==0) break;
		buffer[nread] = '\0';
		str += buffer;
	}
	return TRUE;
}//reciveStr
