/***************************************************************************
                          Hermod.cpp  -  description
                             -------------------
    begin                : Fri Jan 3 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#include "Hermod.h"

time_t garbageColectorLastTurn;
BOOL garbageColectorRunning = FALSE;

#ifdef WIN32
DWORD garbageColector(void *arg)
#else
void *garbageColector(void *arg)
#endif
/*
 * Thread there will colect dead object of the TransactionObj type
 */
{
	ScreenConnectionPool *pool = (ScreenConnectionPool *)arg;
	if (pool==NULL) {
//		pool->logWrite(1,"Object garbage colector got a NULL pool. Thread exited");
		return NULL;
	}

	garbageColectorRunning = TRUE;
	pool->logWrite(1,"Object garbage colector started");
	while (garbageColectorRunning) {
		garbageColectorLastTurn = time(NULL);
		ThreadSleep(60);
		if (!garbageColectorRunning) break;
		pool->collectBlokedConnections();
	}
	garbageColectorRunning = FALSE;
#ifdef WIN32
	//End the thread nicely!
	ExitThread(0);
#endif
	//We never arrive here under WIN32, but it makes the compiler happy:-)
	return NULL;
}//garbageColector



Hermod::Hermod()
{
	m_gbColThread = NULL;
}//Hermod


Hermod::~Hermod()
{
	if (m_gbColThread) {
		garbageColectorRunning = FALSE;
		ThreadSleep(65);
		m_gbColThread->kill();
		delete m_gbColThread;
	}
}//~Hermod


void Hermod::init(char *configFile, BOOL startGarbageColector)
{
	m_connPool.poolConfigure(configFile);
	if (startGarbageColector) goGarbageColector();
}//init


void Hermod::goGarbageColector()
{
	if (m_gbColThread) {
		garbageColectorRunning = FALSE;
		ThreadSleep(65);
		m_gbColThread->kill();
		delete m_gbColThread;
	}
	if ((m_gbColThread=new ThreadClass(garbageColector, (void *)&m_connPool))==NULL) {
		m_connPool.logWrite(2,"Hermod.startGarbageColector() Can not cretate a Garbage Colector thread.");
		return;
	}
	m_gbColThread->start();
}//startGarbageColector


Response *Hermod::invoke(char *xmlStr)
/**
 * This method implement an invoke of a RPC/script.
 * The method take a string as input, the string has
 * to follow the XML-RPC specification as defined by
 * XML-RPC.Com (http://www.xml-rpc.org/spec).
 */
{
	ScreenConnection *conn = NULL;
	Element *xmlTree, *params, *param, *actElm;
	char *methodName, *connectionId, *paramName, *userId, *passwd, errStr[256];
	ConnParameter parameter, *par, **outParams;
	BOOL releaseConnectionAfterUse = FALSE;
	BOOL returnWithoutInvokeOnHost = FALSE;
	MiniXML miniXml;
	Response *response;

	if ((response=new Response())==NULL) return NULL;
	response->clean();
	// Parse XML string
	miniXml.DropTree();
	miniXml.Parse(xmlStr);
	if ((xmlTree=miniXml.GetTree())==NULL) {
		response->createFault(HERMOD_NULL_FAULT, "The sended XML string contain error and can not be parsed");
		return response;
	}

	// Get connection Id if exist
	xmlTree->InitSearch();
	if ((actElm=xmlTree->SearchFor("methodCall"))==NULL) {
		response->createFault(HERMOD_XML_SYNTAX_FAULT, "The sended XML string has no Method Call defined");
		return response;
	}
	if ((connectionId=actElm->GetAttrubuteValue("connection-id"))!=NULL) {
		response->setConnectionId(connectionId);
	}

	// Get method name and parameters
	if ((actElm=xmlTree->SearchFor("methodName"))==NULL || (methodName=actElm->GetValue())==NULL) {
		response->createFault(HERMOD_XML_SYNTAX_FAULT, "The sended XML string has no Method Name defined");
		return response;
	}
	parameter.setScriptName(methodName);
	xmlTree->InitSearch();
	if ((params=xmlTree->SearchFor("params"))==NULL) {
		sprintf(errStr,"The sended XML string has no params tag for the method %s",methodName);
		response->createFault(HERMOD_XML_SYNTAX_FAULT, errStr);
		return response;
	}

	releaseConnectionAfterUse = FALSE;
	returnWithoutInvokeOnHost = FALSE;
	if (strcmp(methodName,"set-user-password")==0) { //somebody is asking to set the user and password for the Mainframe
		param = params->GetFirstSubElement();
		while (param) {
			if ((paramName=param->GetAttrubuteValue("name"))!=NULL) {
				if ((actElm=param->GetFirstSubElement())!=NULL) { //<string>
					if (strcmp(paramName, "user")==0) userId = actElm->GetValue();
					else if (strcmp(paramName, "password")==0) passwd = actElm->GetValue();
				}
			}
			param = param->GetNextElement();
		}
		if (connectionId) conn = m_connPool.findConnection(connectionId);
		else conn = m_connPool.getFreeConnection();
		if (conn) {
			connectionId = conn->getSymbolicConnName();
			parameter.setScriptName("connect");
			parameter.setParamName("user");
			parameter.setValue(userId);
			if (conn->setParameter(&parameter)!=CI_NO_ERR) {
				sprintf(errStr,"There occur an error while setting the user ID: %s",userId);
				response->createFault(HERMOD_WRONG_PARAETER_NAME, errStr);
				conn->realseConnection();
				return response;
			}
			parameter.setParamName("password");
			parameter.setValue(passwd);
			if (conn->setParameter(&parameter)!=CI_NO_ERR) {
				sprintf(errStr,"There occur an error while setting the user ID: %s",userId);
				response->createFault(HERMOD_WRONG_PARAETER_NAME, errStr);
				conn->realseConnection();
				return response;
			}
		}
		response->setConnectionId(connectionId);
//		releaseConnectionAfterUse = TRUE;
		returnWithoutInvokeOnHost = TRUE;
	}
	// Get/release a ScreenConnection
	else if (strcmp(methodName,"get-free-connection")==0) { //sombody is asking for a reservation of a free connection
		conn = m_connPool.getFreeConnection();
		if (conn) connectionId = conn->getSymbolicConnName();
		response->setConnectionId(connectionId);
		returnWithoutInvokeOnHost = TRUE;
	}
	else if (strcmp(methodName,"get-connection")==0) { //sombody is asking for a reservation of a particularly connection
		conn = m_connPool.getConnection(connectionId);
		response->setConnectionId(connectionId);
		returnWithoutInvokeOnHost = TRUE;
	}
	else if (strcmp(methodName,"release-connection")==0) {//somebody is going to release a ealier reservated connetion
		conn = m_connPool.findConnection(connectionId);
		if (conn) conn->realseConnection();
		response->setConnectionId(connectionId);
		returnWithoutInvokeOnHost = TRUE;
	}
	else if (connectionId) conn = m_connPool.findConnection(connectionId); //sombody is asking to use a connection they has reservated ealier
	else { //find the first free connection an use it.
		conn = m_connPool.getFreeConnection();
		releaseConnectionAfterUse = TRUE;
	}
if (connectionId) printf("%s has Connection ID=%s\n",methodName,connectionId);
else printf("%s has No Connection ID\n",methodName);
	
	if (returnWithoutInvokeOnHost) {
		response->createResponse();
		if (releaseConnectionAfterUse && conn!=NULL) conn->realseConnection();
		return response;
	}
	
	if (conn==NULL) {
		char faultStr[256];
		if (connectionId==NULL) sprintf(faultStr,"The connection ID (NULL) was wrong or there was no free connections");
		else sprintf(faultStr,"The connection ID (%s) was wrong or there was no free connections",connectionId);
		response->createFault(HERMOD_NULL_CONNECTION_FAULT, faultStr);
		return response;
	}

	// Set parameters/variables
	params->InitSearch();
	param = params->GetFirstSubElement();
	while (param) {
		if ((paramName=param->GetAttrubuteValue("name"))==NULL) {
			conn->realseConnection();
			response->createFault(HERMOD_XML_SYNTAX_FAULT, "The sended XML string missing a name attribute in the param tag");
			return response;
		}
		parameter.setParamName(paramName);
		if ((actElm=param->GetFirstSubElement())==NULL) { //<string>
			conn->realseConnection();
			sprintf(errStr,"The sended XML string missing a scalar tag in the value tag for the param: %s",paramName);
			response->createFault(HERMOD_XML_SYNTAX_FAULT, errStr);
			return response;
		}
		parameter.setValue(actElm->GetValue());
		if (conn->setParameter(&parameter)!=CI_NO_ERR) {
			conn->realseConnection();
			sprintf(errStr,"The sended XML string contain wrong parameter names or values: <param name='%s'><value><string>%s",paramName,actElm->GetValue());
			response->createFault(HERMOD_WRONG_PARAETER_NAME, "");
			return response;
		}

		param = param->GetNextElement();
	}

	// Do the method/command
	if (!conn->isConnected()) {
		if (conn->openConnection()!=CI_NO_ERR) {
			response->createFault(HERMOD_METHOD_EXECUTING_ERROR, "The Open Connection method returned with error");
			if (releaseConnectionAfterUse) conn->realseConnection();
			return response;
		}
	}
	if (conn->doCommand(methodName)!=CI_NO_ERR) {
		conn->realseConnection();
		sprintf(errStr,"There occur an error while exeicuting the method: %s or the method does not exist",methodName);
		response->createFault(HERMOD_METHOD_EXECUTING_ERROR, errStr);
		if (releaseConnectionAfterUse) conn->realseConnection();
		return response;
	}

	// Get output parameters
	if ((outParams=conn->getAllOutputParameters(methodName))!=NULL) {
		int i=0;
		do {
			par = outParams[i];
			response->insertParam(par);
			if (par) delete par; //clean up!
			i++;
		} while (par!=NULL);
		delete []outParams; //clean up!
	}

	response->createResponse();

	if (releaseConnectionAfterUse) conn->realseConnection();
	return response;
}//invoke





Response::Response()
{
	m_firstParam = m_lastParam = NULL;
	m_connectionId = NULL;
}//Response


Response::~Response()
{
	clean();
}//~Response


void Response::clean()
{
	ResponseParam *actParm, *delParm;

	actParm = m_firstParam;
	while (actParm) {
		delParm = actParm;
		actParm = actParm->next;
		delete delParm;
	}
	m_firstParam = m_lastParam = NULL;
	if (m_connectionId) delete []m_connectionId;
	m_connectionId = NULL;
}//clean


void Response::insertParam(ConnParameter *param)
{
	ResponseParam *actParm;

	if (param==NULL) return;

	if ((actParm=new ResponseParam())==NULL) return;
	*actParm = *param;
	if (m_lastParam==NULL) {
		m_firstParam = m_lastParam = actParm;
	}
	else {
		m_lastParam->next = actParm;
		m_lastParam = actParm;
	}
}//insertParam


void Response::createResponse()
{
	Element *xmlTree, *actElm, *subElm, *paramsElm;
	ElementAttribute *attr;
	ResponseParam *actParm;

	m_miniXML.DropTree();
	xmlTree = actElm = m_miniXML.GetTree();

	if ((subElm=new Element())==NULL) return;
	subElm->SetName("methodResponse");
	if (m_connectionId) {
		if ((attr=new ElementAttribute())==NULL) return;
		if ((attr->mName=(char *)malloc(15))==NULL) return; strcpy(attr->mName,"connection-id");
		if ((attr->mValue=(char *)malloc(strlen(m_connectionId)+1))==NULL) return; strcpy(attr->mValue, m_connectionId);
		subElm->AddAttribute(attr);
	}
	actElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm=new Element())==NULL) return;
	subElm->SetName("params");
	actElm->AddSubElement(subElm);
	paramsElm = actElm = subElm;

	actParm = m_firstParam;
	while (actParm) {
		if ((subElm = new Element())==NULL) return;
		subElm->SetName("param");
		if ((attr = new ElementAttribute())==NULL) return;
		if ((attr->mName=(char *)malloc(15))==NULL) return; strcpy(attr->mName,"name");
		if ((attr->mValue=(char *)malloc(strlen(actParm->paramName)+1))==NULL) return; strcpy(attr->mValue, actParm->paramName);
		subElm->AddAttribute(attr);
		paramsElm->AddSubElement(subElm);
		actElm = subElm;

		if ((subElm = new Element())==NULL) return;
		subElm->SetName("value");
		actElm->AddSubElement(subElm);
		actElm = subElm;

		if ((subElm = new Element())==NULL) return;
		subElm->SetName("string");
		subElm->SetValue(actParm->value);
		actElm->AddSubElement(subElm);

		actParm = actParm->next;
	}

}//createResponse


void Response::createFault(int faultCode, char *faultStr)
{
	Element *xmlTree, *actElm, *subElm, *struElm, *memElm;
	char faultCodeStr[10];

	sprintf(faultCodeStr,"%d",faultCode);

	m_miniXML.DropTree();
	xmlTree = actElm = m_miniXML.GetTree();

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("methodResponse");
	actElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("fault");
	actElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("value");
	actElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("struct");
	actElm->AddSubElement(subElm);
	struElm = actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("member");
	struElm->AddSubElement(subElm);
	memElm = actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("name");
	subElm->SetValue("faultCode");
	memElm->AddSubElement(subElm);

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("value");
	memElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("int");
	subElm->SetValue(faultCodeStr);
	actElm->AddSubElement(subElm);

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("member");
	struElm->AddSubElement(subElm);
	memElm = actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("name");
	subElm->SetValue("faultString");
	memElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("value");
	memElm->AddSubElement(subElm);
	actElm = subElm;

	if ((subElm = new Element())==NULL) return;
	subElm->SetName("string");
	subElm->SetValue(faultStr);
	actElm->AddSubElement(subElm);

	
}//fault

char *Response::getXmlString()
{
	m_miniXML.InitOutStr();
	m_miniXML.Tree2Str(m_miniXML.GetTree());
	return m_miniXML.GetOutStr();
}//getXmlString
