/***************************************************************************
                          Hermod.h  -  description
                             -------------------
    begin                : Fri Jan 3 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _HERMOD_H_

#include "ScreenConnection/ScreenConnection.h"
#include "MiniXml/MiniXML.h"


#define HERMOD_NULL_FAULT 100
#define HERMOD_XML_SYNTAX_FAULT 101
#define HERMOD_NULL_CONNECTION_FAULT 102
#define HERMOD_WRONG_PARAETER_NAME 103
#define HERMOD_WRONG_METHOD_NAME 104
#define HERMOD_METHOD_EXECUTING_ERROR 105

class Response;

class Hermod
{
	private:
		MiniXML m_miniXML;
		ScreenConnectionPool m_connPool;
		ThreadClass *m_gbColThread;

	public:
		Hermod();
		~Hermod();

		void logWrite(int level, const char *message) {m_connPool.logWrite(level, message);};
		void init(char *configFile, BOOL startGarbageColector=TRUE);
		void goGarbageColector();
		Response *invoke(char *xmlStr);
		ScreenConnectionPool *getConnectionPool() {return &m_connPool;};
		void setUserPassword(char *user, char *password) {m_connPool.setUserPassword(user, password);};
};//Hermod


class ResponseParam
{
	public:
		char *paramName;
		char *value;
		ResponseParam *next;

		ResponseParam() {paramName=NULL; value=NULL; next = NULL;};
		~ResponseParam() {if (paramName) delete []paramName; if (value) delete []value;};
		void setParamName(char *name) {if (paramName) delete []paramName; paramName=NULL; if (name) {if ((paramName=new char[strlen(name)+1])==NULL) return; strcpy(paramName,name);}};
		void setValue(char *val) {if (value) delete []value; value=NULL; if (val) {if ((value=new char[strlen(val)+1])==NULL) return; strcpy(value,val);}};

		ResponseParam& operator = (const ConnParameter& p) {setParamName(p.paramName); setValue(p.value); return *this;};
};//ResponseParam


class Response
{
	private:
		MiniXML m_miniXML;
		ResponseParam *m_firstParam, *m_lastParam;
		char *m_connectionId;

	public:
		Response();
		~Response();
		void clean();

		void setConnectionId(char *id) {if (m_connectionId) delete []m_connectionId; m_connectionId=NULL; if(id) {if ((m_connectionId=new char[strlen(id)+1])==NULL) return; strcpy(m_connectionId,id);}};
		char *getConnectionId() {return m_connectionId;}
		void insertParam(ConnParameter *param);
		void createResponse();
		void createFault(int faultCode, char *faultStr);
		char *getXmlString();
};//Response


#endif
