/***************************************************************************
                          HermodBind.cpp  -  description
                             -------------------
    begin                : Fri Jan 3 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#include "Hermod.h"
#include "MiniHttp/MiniHTTP.h"

extern BOOL garbageColectorRunning;

Hermod *hermodObj = NULL;
MiniHTTP miniHttpObj;


void HttpGetCall(char *httpMsgBody, HttpSession *httpSession)
/*
 * This is the function there is called when a HTTP POST method
 * is invorked. The httpMsgBody parameter point to the body of
 * the http message. The responseMsg is a pointer to a char
 * pointer where the call can allocate and respond with a message.
 */
{
	Response *response;
	char *xmlOutStr;

	try {
		response = hermodObj->invoke(httpMsgBody);
	} catch (...) {
		char headerStr[50], bodyStr[256];
		strcpy(headerStr, "500 Internal Server Error");
		sprintf(bodyStr,"<HTML><HEAD></HEAD<BODY>\n<H1>Error 500</H1>\nInternal Server Error \n</BODY></HTML>\n");
		httpSession->writeError(headerStr, bodyStr);
		miniHttpObj.logWrite(1,"EXCEPTION... in HttpGetCall()");
		return;
	}
	if (hermodObj==NULL || response==NULL) {
		char headerStr[50], bodyStr[256];
		strcpy(headerStr, "500 Internal Server Error");
		sprintf(bodyStr,"<HTML><HEAD></HEAD<BODY>\n<H1>Error 500</H1>\nInternal Server Error \n</BODY></HTML>\n");
		httpSession->writeError(headerStr, bodyStr);
		return;
	}
	xmlOutStr = response->getXmlString();

	httpSession->setContentLength(strlen(xmlOutStr));
	httpSession->setContentType("text/xml");
	httpSession->writeOkHeader();
	httpSession->sendStr(xmlOutStr);
	delete response;
}//HttpGetCall


int HttpPostCall(char *httpMsgBody, char **responseMsg, HttpSession *httpSession)
/*
 * This is the function there is called when a HTTP POST method
 * is invorked. The httpMsgBody parameter point to the body of
 * the http message. The responseMsg is a pointer to a char
 * pointer where the call can allocate and respond with a message.
 * If the return value is diffrent from 0 the responseMsg
 * contain a error message.
 */
{
	Response *response;
	char *xmlOutStr;

printf("HttpPostCall()\n");
	try {
		response=hermodObj->invoke(httpMsgBody);
	} catch (...) {
		*responseMsg = (char *)malloc(256);
		strcpy(*responseMsg, "500 Internal Server Error");
		miniHttpObj.logWrite(1,"EXCEPTION... in HttpPostCall()");
		return 500;
	}
	if (hermodObj==NULL || response==NULL) {
		*responseMsg = (char *)malloc(256);
		strcpy(*responseMsg, "500 Internal Server Error");
		return 500;
	}
	xmlOutStr = response->getXmlString();
printf("xmlOutStr=\n%s\n",xmlOutStr);

	*responseMsg = (char *)malloc(strlen(xmlOutStr)+1);
	strcpy(*responseMsg, xmlOutStr);
	httpSession->setContentType("text/xml");
	delete response;
	return 0;
}//HttpPostCall




int HermodBind(char *configFile, char *user, char *password, short httpPort, char *httpLogFile)
/*
 * This is the actual program/thread for the service
 */
{
	ScreenConnectionPool *pool;

	miniHttpObj.openLog(httpLogFile, 4);
	if (miniHttpObj.getHaveError()) {
		miniHttpObj.logWrite(1,miniHttpObj.getErrorString());
		printf("ERROR: %s\n",miniHttpObj.getErrorString());
		return 1;
	}
	if ((hermodObj = new Hermod())==NULL) return 1;
	hermodObj->init(configFile);
	hermodObj->setUserPassword(user, password);

	miniHttpObj.setGetFunctionCall(HttpGetCall);
	miniHttpObj.setPostFunctionCall(HttpPostCall);

	// Now run the service!
	miniHttpObj.runService(httpPort);
	if (miniHttpObj.getHaveError()) {
		miniHttpObj.logWrite(1,miniHttpObj.getErrorString());
		printf("ERROR: %s\n",miniHttpObj.getErrorString());
	}

	hermodObj->logWrite(1,"Service is going down now!");

	// Block pool!
	pool = hermodObj->getConnectionPool();
	pool->setPoolGoingDown(1);

	garbageColectorRunning = FALSE; // Try to stop the Garbage Colector graceful
	ThreadSleep(60);
	hermodObj->logWrite(1,"The Hermod object is going to be deleted now!");
	delete hermodObj;

	return 0;
}//HermodBind


void StopHermod(short port)
{
	struct sockaddr_in serv_addr;
	SOCKET sockfd;

	miniHttpObj.logWrite(1, "Got stop signal. Wait to I have shutdown all threads");
	miniHttpObj.stopMiniHttp();

	// Connect to http server to stop it!
	memset((void *)&serv_addr, 0, sizeof(struct sockaddr_in));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);
	serv_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) return;
	if (connect(sockfd, (const struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) return;
#ifdef LINUX
	close(sockfd);
#elif defined(WIN32)
	closesocket(sockfd);
#endif
	ThreadSleep(30);
}// StopHermod

