/***************************************************************************
                          HttpLog.cpp  -  description
                             -------------------
    begin                : Mon June 24 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/
#include <time.h>
#include <string.h>
#include "HttpLog.h"

HttpLog::HttpLog()
{
	m_logLevel = 1;
	m_isOpen = FALSE;
	m_openDay = 0;
#ifdef LINUX
	pthread_mutexattr_init(&m_mutex_attr);
	pthread_mutex_init(&m_mutex, &m_mutex_attr);
#elif defined(WIN32)
	InitializeCriticalSection(&m_mutex);
#endif
}//HttpLog

HttpLog::HttpLog(const char *logName)
{
	m_logLevel = 1;
	m_isOpen = FALSE;
	m_openDay = 0;
#ifdef LINUX
	pthread_mutexattr_init(&m_mutex_attr);
	pthread_mutex_init(&m_mutex, &m_mutex_attr);
#elif defined(WIN32)
	InitializeCriticalSection(&m_mutex);
#endif
	open(logName);
}//HttpLog

HttpLog::~HttpLog()
{
#ifdef LINUX
	pthread_mutex_destroy(&m_mutex);
	pthread_mutexattr_destroy(&m_mutex_attr);
#elif defined(WIN32)
	DeleteCriticalSection(&m_mutex);
#endif
	m_oStream.close();
}//~HttpLog

void HttpLog::open(const char *logName)
{
	time_t lTime;
	struct tm *tim;

#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	if (m_isOpen) m_oStream.close();

	m_oStream.open(logName, ios::app);

	filebuf *fb = m_oStream.rdbuf();
	if (fb->is_open()) {
		strncpy(m_logName, logName, 1024);
		m_logName[1024] = '\0';
		m_isOpen = TRUE;
	}
	else {
		*m_logName = '\0';
		m_isOpen = FALSE;
	}
	time(&lTime);
	tim = gmtime(&lTime);
	m_openDay = tim->tm_mday;
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
}//open

void HttpLog::close()
{
#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	if (m_isOpen) m_oStream.close();
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
	m_isOpen = FALSE;
}//close

void HttpLog::logWrite(const char *string)
{
	logWrite(1,string);
}//logLevel

void HttpLog::logWrite(int level, const char *string)
{
	time_t lTime;
	char timeStr[80];
	struct tm *tim;

	if (level > m_logLevel) return;

#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	time(&lTime); strcpy(timeStr,ctime(&lTime)); timeStr[strlen(timeStr)-1] = '\0';
	tim = gmtime(&lTime);
	if (tim->tm_mday != m_openDay) {
		m_oStream.close();
		m_isOpen = FALSE;
		rotateLogs();
		open(m_logName);
	}
	m_oStream << level << " <" << timeStr << "> " << string << endl;
	m_oStream.flush();
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
}//logWrite


void HttpLog::rotateLogs()
{
	char logFile[1030], destFile[1030];

	sprintf(logFile,"%s.9",m_logName);
	remove(logFile);
	for (int i=8; i>0; i--) {
		sprintf(logFile,"%s.%d",m_logName,i);
		sprintf(destFile,"%s.%d",m_logName,(i+1));
		rename(logFile, destFile);
	}
	sprintf(destFile,"%s.1",m_logName);
	rename(m_logName, destFile);
}//rotateLogs
