/***************************************************************************
                          MiniHTTP.cpp  -  description
                             -------------------
    begin                : Fri May 4 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/

#include <signal.h>
#include "MiniHTTP.h"


#define MAX_CONN_TIME 6000


#ifdef LINUX
void *miniHttpGarbageColector(void *arg)
#elif defined(WIN32)
DWORD miniHttpGarbageColector(void *arg)
#endif
/*
 * Thread there will colect sessions
 */
{
	MiniHTTP *miniHTTP = (MiniHTTP *)arg;
	HttpSession *actSess, *delSess;
	time_t nowTime;

	if (miniHTTP==NULL) return NULL;

	miniHTTP->logWrite(1,"HTTP Garbage Colector is Running");
	while (miniHTTP->isGarbageColectorRunning()) {
#ifdef LINUX
		sleep(60);
#elif defined (WIN32)
		Sleep(60000);
#endif
		nowTime = time(NULL);
		actSess = miniHTTP->getFirstSession();
		miniHTTP->lockHttp();
		while (actSess) {

			if (actSess->getTouchTime()+MAX_CONN_TIME < nowTime) delSess = actSess;
			else delSess = NULL;
			actSess = actSess->getNextSession();
			if (delSess) {
				miniHTTP->logWrite(1,"HTTP Garbage Colector deleting session");
				try {
					delete delSess;
				} catch (...) {
					miniHTTP->logWrite(1,"EXCEPTION... in miniHttpGarbageColector()");
				}
			}
		}
		miniHTTP->unlockHttp();
	}

	miniHTTP->logWrite(1,"HTTP Garbage Colector is dieing\n");
#ifdef WIN32
	//End the thread nicely!
	ExitThread(0);
#endif
	//We never arrive here under WIN32, but it makes the compiler happy:-)
	return NULL;
}//miniHttpGarbageColector



MiniHTTP::MiniHTTP()
{
	m_firstSession = m_lastSession = NULL;
	m_miniHttpGarbageColectorIsRunning = 1;
	*m_errorString = '\0';
	m_haveError = 0;
#if LINUX
	// Setup a mutext to control of critical regions
	pthread_mutexattr_t mutex_attr;
	pthread_mutexattr_init(&mutex_attr);
	pthread_mutex_init(&m_mutex, &mutex_attr);
	pthread_mutexattr_destroy(&mutex_attr);
	
	// Start the garbage colector of blocked socket.
	pthread_attr_t threadAttr;
	pthread_attr_init(&threadAttr);
	int r = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
	r = pthread_create(&m_threadId, &threadAttr, miniHttpGarbageColector, (void *) this);
	pthread_attr_destroy(&threadAttr);

	// We do not want taht the application terminate if a client close a socket before
	// we are finish using it.
	sigset_t signalSet;
	if (sigaddset(&signalSet, SIGPIPE) < 0) {
		strcpy(m_errorString,"Server can't set signalset to SIGPIPE");
		m_haveError = 1;
	}
	else {
		if (sigprocmask(SIG_BLOCK, &signalSet, NULL) < 0) {
			strcpy(m_errorString,"Server can't process the siganl set (SIGPIPE)");
			m_haveError = 1;
		}
	}
#elif defined(WIN32)
	InitializeCriticalSection(&m_mutex);
	DWORD id;
	m_threadId = CreateThread(0, 0, (LPTHREAD_START_ROUTINE) miniHttpGarbageColector, (void *)this, 0, &id);
#endif
	strcpy(m_homeDir, ".");
	m_log = NULL;
	m_postFunctionCall = NULL;
	m_stopMiniHTTP = 0;
	m_soccketFd = -1;
	logWrite(1,"MiniHTTP object was created");
}//MiniHTTP


MiniHTTP::~MiniHTTP()
{
	m_miniHttpGarbageColectorIsRunning = 0;
	cleanup();
#ifdef LINUX
	pthread_mutex_destroy(&m_mutex);
#elif defined(WIN32)
	DeleteCriticalSection(&m_mutex);
#endif
	logWrite(1,"MiniHTTP object was deleted");
	delete m_log;
}//~MiniHTTP


void MiniHTTP::openLog(char *logName, int logLevel)
{
	if (m_log) delete m_log;
	if ((m_log=new HttpLog(logName))==NULL) return;
	m_log->setLogLevel(logLevel);
}//openLog


void MiniHTTP::cleanup()
{
	HttpSession *actSess, *delSess;
#ifdef LINUX
	pthread_t  threadId;
#elif defined(WIN32)
	HANDLE threadId;
#endif

#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	actSess = m_firstSession;
	while (actSess) {
		delSess = actSess;
		actSess = actSess->getNextSession();

		//Clean up!
#ifdef LINUX
		threadId =  delSess->getThreadId();
		if (threadId > 0) pthread_kill(threadId, SIGKILL);
#elif defined(WIN32)
		threadId =  delSess->getThreadId();
		if (threadId > 0) CloseHandle(threadId);
#endif
		delete delSess;
	}
#ifdef LINUX
	if (m_soccketFd > 0) close(m_soccketFd);
#elif defined(WIN32)
	if (m_soccketFd > 0) closesocket(m_soccketFd);
#endif
	shutdown(m_soccketFd, 2);
	m_soccketFd = -1;
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
}


int MiniHTTP::runService(short port)
{
	int clientLen;
	struct sockaddr_in  serverAddr, clientAddr;
	SOCKET clientSocketFd;
	HttpSession *newSession;

	// Open a TCP socket
	if ((m_soccketFd=socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		sprintf(m_errorString,"Server can't open stream socket");
		m_haveError = 1;
		return 0;
	}

	// Bind put local address so the client can send to us
	memset((void *)&serverAddr, 0, sizeof(struct sockaddr_in));
	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	serverAddr.sin_port = htons(port);

	if (bind(m_soccketFd, (struct sockaddr *) &serverAddr, sizeof(struct sockaddr_in)) < 0) {
		sprintf(m_errorString,"Server can't bind local address");
		m_haveError = 1;
		return 1;
	}

	if (listen(m_soccketFd, 5)!=0) {
		sprintf(m_errorString,"Server can't listen");
		m_haveError = 1;
		return 1;
	}

	logWrite(1, "MiniHttp is ready for data\n");
printf("MiniHTTP ready for data!\n");
	while (!m_stopMiniHTTP) {
		// Wait for a client to contact us
		clientLen = sizeof(struct sockaddr_in);
#ifdef LINUX
		clientSocketFd = accept(m_soccketFd, (struct sockaddr *) &clientAddr, (socklen_t *)&clientLen);
#elif defined(WIN32)
		clientSocketFd = accept(m_soccketFd, (struct sockaddr *) &clientAddr, &clientLen);
#endif
printf("MiniHTTP connection recived\n");
		if (m_stopMiniHTTP) break;
		if (clientSocketFd < 0) {
			sprintf(m_errorString,"Server accept error");
			m_haveError = 1;
			continue;
		}

		// Create a new session for the client and insert it the session queue
		if ((newSession=new HttpSession(this, &clientAddr, clientLen))==NULL) {
#ifdef LINUX
			close(clientSocketFd);
#elif defined(WIN32)
			closesocket(clientSocketFd);
#endif
			continue;
		}
		insertSession(newSession);

		// Let the session take over the control of the client
		newSession->runSession(clientSocketFd);
	}
#ifdef LINUX
	close(m_soccketFd);
#elif defined(WIN32)
	closesocket(m_soccketFd);
#endif
	shutdown(m_soccketFd, 2);
	m_soccketFd = -1;

	return 0;
}//runService


void MiniHTTP::insertSession(HttpSession *sess)
{
#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	if (m_firstSession==NULL) {
		m_firstSession = m_lastSession = sess;
	}
	else {
		m_lastSession->setNextSession(sess);
		m_lastSession = sess;
	}
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
}//insertSession


void MiniHTTP::deleteSession(HttpSession *sess)
{
	HttpSession *actSess, *nextSess;

	if (sess==NULL) return;

#ifdef LINUX
	pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
#endif
	if (sess==m_firstSession) {
		m_firstSession = m_firstSession->getNextSession();
		if (sess==m_lastSession) m_lastSession = m_firstSession;
#ifdef LINUX
		pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
		LeaveCriticalSection(&m_mutex);
#endif
		return;
	}
	actSess = m_firstSession;
	while (actSess) {
		if ((nextSess=actSess->getNextSession())==sess) {
			actSess->setNextSession(nextSess->getNextSession());
			if (sess==m_lastSession) m_lastSession = actSess;
#ifdef LINUX
			pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
			LeaveCriticalSection(&m_mutex);
#endif
			return;
		}
		actSess = actSess->getNextSession();
	}
#ifdef LINUX
	pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
#endif
}//deleteSession


int MiniHTTP::lockHttp()
{
#ifdef LINUX
	return pthread_mutex_lock(&m_mutex);
#elif defined(WIN32)
	EnterCriticalSection(&m_mutex);
	return 0;
#endif
}//lockHttp


int MiniHTTP::unlockHttp()
{
#ifdef LINUX
	return pthread_mutex_unlock(&m_mutex);
#elif defined(WIN32)
	LeaveCriticalSection(&m_mutex);
	return 0;
#endif
}//unlockHttp


