/***************************************************************************
                          Element.cpp  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU LGPL (Lesser general Public License) 
 *   as published by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/lesser.html or in the file copying.txt
 *
 ***************************************************************************/
#include "Element.h"

//int GlobalCreateCounter = 0;

Element::Element()
{
	mName = NULL;
	mValue = NULL;
	mFirstAttribute = NULL;
	mLastAttribute = NULL;
	mNextElement = NULL;
	mFirstSubElement = NULL;
	mLastSubElement = NULL;
	mActualeSubElement = NULL;
	mParrent = NULL;
	mSearchFound = 0;
//	GlobalCreateCounter++;
//	printf("Create %i\n",GlobalCreateCounter);
}//Element

Element::~Element()
{
	DeleteAttrubutes();
	if (mName) free(mName);
	mName = NULL;
	if (mValue) free(mValue);
	mValue = NULL;
//	printf("Destroy %i\n",GlobalCreateCounter);
//	GlobalCreateCounter--;
}//~Element

/*void Element::DeleteSubElements()
{
	Element *subElm, *nextElm, *clearElm;

/*	subElm = GetFirstSubElement();
	if (subElm) {
		subElm->DeleteSubElements();
		delete subElm;
	}

	nextElm = GetNextElement();
	while (nextElm) {
		if ((subElm=nextElm->GetFirstSubElement())!=NULL) subElm->DeleteSubElements();
//		nextElm->DeleteSubElements();
		clearElm = nextElm;
		nextElm = nextElm->GetNextElement();
		delete clearElm;
	}
	if ((subElm=this->GetFirstSubElement())!=NULL) subElm->DeleteSubElements();
	delete this;
}//DeleteElements
*/

void Element::DeleteNextElements()
{
	Element *nextElm, *clearElm;

	nextElm = GetNextElement();
	while (nextElm) {
		if (nextElm->GetFirstSubElement()!=NULL) {
			nextElm->DeleteSubElements();
		}
		clearElm = nextElm;
		nextElm = nextElm->GetNextElement();
		delete clearElm;
	}
	mNextElement = NULL;
}//DeleteNextElements

void Element::DeleteSubElements()
{
	Element *subElm;


	subElm = GetFirstSubElement();
	if (subElm) {
		subElm->DeleteNextElements();
		subElm->DeleteSubElements();
		delete subElm;
	}
	mFirstSubElement = mLastSubElement = NULL;
}//DeleteSubElements


void Element::CutOutOfTree()
{
	Element *elm;

	if (mParrent==NULL) {
		mNextElement = NULL;
		return;
	}
	if (mParrent->mFirstSubElement==this) {
		if (mParrent->mLastSubElement==this) {
			mParrent->mFirstSubElement = mParrent->mLastSubElement = NULL;
			mNextElement = NULL;
			return;
		}
		mParrent->mFirstSubElement = mNextElement;
		mNextElement = NULL;
		return;
	}
	elm = mParrent->mFirstSubElement;
	while (elm) {
		if (elm->mNextElement==this) break;
		elm = elm->mNextElement;
	}
	if (elm) {
		elm->mNextElement = mNextElement;
		if (mParrent->mLastSubElement==this) mParrent->mLastSubElement = elm;
	}
	mNextElement = NULL;
}//CutOutOfTree

void Element::SetName(char *name)
{
	if (name==NULL) return;
	if (mName) free(mName);
	if((mName=(char *)malloc(strlen(name)+1))==NULL) return;
	strcpy(mName,name);
}//SetName

void Element::SetValue(char *value)
{
	if (value==NULL) return;
	if (mValue) free(mValue);
	if((mValue=(char *)malloc(strlen(value)+1))==NULL) return;
	strcpy(mValue,value);
}//SetValue

void Element::AddAttribute(ElementAttribute *attr)
{
	if (mFirstAttribute==NULL) {
		mFirstAttribute = attr;
		mLastAttribute = attr;
		attr->mNext = NULL;
		return;
	}

	mLastAttribute->mNext = attr;
	attr->mNext = NULL;
	mLastAttribute = attr;
}//AddAttribute

void Element::DeleteAttrubutes()
{
	ElementAttribute *actAttr, *clearAttr;

	actAttr = mFirstAttribute;
	while (actAttr) {	
		clearAttr = actAttr;
		actAttr = actAttr->mNext;
		delete clearAttr;
	}
	mFirstAttribute = NULL;
	mLastAttribute = NULL;
}//DeleteAttrubutes

char *Element::GetAttrubuteValue(char *attrName)
{
	ElementAttribute *attrPtr;

	if (mFirstAttribute==NULL) return NULL;
	if (attrName==NULL) return NULL;
	attrPtr =  mFirstAttribute;
	while (attrPtr) {
		if (attrPtr->mName!=NULL) {
			if (strcmp(attrName, attrPtr->mName)==0) return attrPtr->mValue;
		}
		attrPtr = attrPtr->mNext;
	}
	return NULL;
}//GetAttrubuteValue

void Element::AddSubElement(Element *elm)
{
	if (elm==NULL) return;

	if (mFirstSubElement==NULL) {
		mFirstSubElement = elm;
		mLastSubElement = elm;
		elm->mNextElement = NULL;
		elm->SetParrent(this);
		return;
	}
	mLastSubElement->mNextElement = elm;
	elm->mNextElement = NULL;
	mLastSubElement = elm;
	elm->SetParrent(this);
}//AddSubElement

void Element::AddSubElementFirst(Element *elm)
{
	if (elm==NULL) return;

	if (mFirstSubElement==NULL) {
		mFirstSubElement = elm;
		mLastSubElement = elm;
		elm->mNextElement = NULL;
		elm->SetParrent(this);
		return;
	}
	elm->mNextElement = mFirstSubElement;
	mFirstSubElement = elm;
	elm->SetParrent(this);
}//AddSubElementFirst

void Element::InitSearch()
{
	Element *subElm;

	mSearchFound = 0;
	subElm = GetFirstSubElement();
	while (subElm) {
		subElm->InitSearch();
		subElm = GetNextSubElement();
	 }
}//InitSearch

Element *Element::SearchFor(char *name)
{
	Element *subElm;

	if (name==NULL) return NULL;

	if (mName!=NULL) {
		if (strcmp(name, mName)==0 && !mSearchFound) {
			mSearchFound = 1;
			return this;
		}
	}
	subElm = GetFirstSubElement();
	while (subElm) {
		subElm = subElm->SearchFor(name);
		if (subElm!=NULL) return subElm;
		subElm = GetNextSubElement();
	}
	return NULL;
}//SearchFor
