/***************************************************************************
                          ConnectionInterface.h  -  description
                             -------------------
    begin                : Mon May 13 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _CONNECTIONINTERFACE_H_
#define _CONNECTIONINTERFACE_H_

#if defined (USE_PERCOMM) || (USE_EXEMUL)
 #ifdef WIN32
  #ifndef _WINDOWS_H_INCLUDED_
   #define _WINDOWS_H_INCLUDED_
   #include <windows.h>
  #endif
 #endif
#elif defined(USE_X3270)
 #ifdef LINUX
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
  #include <unistd.h>
  #include <string.h>
 #endif
#endif

#ifndef _MAX_PATH
#define _MAX_PATH 1024
#endif

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

// Return codes
#define CI_NO_ERR 0
#define CI_ERR_NULL_CONN_ID -1
#define CI_ERR_NET_STARTUP -2
#define CI_ERR_HOST_NAME -3
#define CI_ERR_OPEN_SOCKET -4
#define CI_ERR_CONNECT -5
#define CI_ERR_BIND -6
#define CI_ERR_LISTEN -7
#define CI_ERR_ACCEPT -8;
#define CI_ERR_WRITE -9
#define CI_ERR_READ -10
#define CI_ERR_TIMEOUT -11
#define CI_ERR_PROTOCOL -12
#define CI_ERR_PROTOCOL_BODY -13
#define CI_ERR_PACKET -14
#define CI_ERR_INIT -20
#define CI_ERR_ALLOCATE -21
#define CI_ERR_DEALLOCATE -29
#define CI_ERR_OPENHLLAPI -31
#define CI_ERR_NOHLLAPIFUNC -32
#define CI_ERR_NOT_IMPLEMENTED -100

#define CI_BLOCKED -1


struct _ConnectionId
{
#if defined (USE_PERCOMM)
	char sessionName;
	char connSessionName;
	char scriptsFileName[_MAX_PATH];
	char wsProfileFileName[_MAX_PATH];
//	char emulCommand[_MAX_PATH];
#elif defined(USE_EXEMUL)
	char sessionName;
	char connSessionName;
	char scriptsFileName[_MAX_PATH];
	char emulCommand[_MAX_PATH];
#elif defined(USE_X3270)
	char hostName[_MAX_PATH];
	short port;
	char fifoFileName[_MAX_PATH];
	char emulCommand[_MAX_PATH];
	char scriptsFileName[_MAX_PATH];
#endif
	char symbolicConnName[65];
	int timeOutSec;
};
typedef struct _ConnectionId ConnectionId;


class ConnParameter
{
	public:
		char *scriptName;
		char *paramName;
		char *value;

		ConnParameter() {scriptName=NULL;paramName=NULL;value=NULL;};
		~ConnParameter() {if (scriptName) delete []scriptName; if (paramName) delete []paramName; if(value) delete []value;};
		void setScriptName(char *name) {if (scriptName) delete []scriptName; scriptName=NULL; if (name) {scriptName=new char[strlen(name)+1]; strcpy(scriptName,name);}};
		void setParamName(char *name) {if (paramName) delete []paramName; paramName=NULL; if (name) {paramName=new char[strlen(name)+1]; strcpy(paramName,name);}};
		void setValue(char *val) {if (value) delete []value; value=NULL; if (val) {value=new char[strlen(val)+1]; strcpy(value,val);}};
};//ConnParameter


class ConnectionInterface
{
	public:
		virtual int openConnection(ConnectionId *id) {return CI_ERR_NOT_IMPLEMENTED;};
		/**
		 * This function will open a connection for a client application to the server.
		 * Return value:
		 *   0 if the connection was opened else an error code.
		 */
		virtual int closeConnection(ConnectionId *id) {return CI_ERR_NOT_IMPLEMENTED;};
		/**
		 * Close and release the resource for a connection.
		 * Return value:
		 *   0 if the connection was closed else an error code.
		 */
		virtual int setParameter(ConnParameter *param)  {return CI_ERR_NOT_IMPLEMENTED;};
		/**
		 * Set a parameter to a value.
		 * Return value:
		 *   0 if the variable was set else an error code.
		 */
		virtual int getParameter(ConnParameter *param) {return CI_ERR_NOT_IMPLEMENTED;};
		/**
		 * Get a parameter to a value.
		 * Return value:
		 *   0 if the variable was set else an error code.
		 */
		virtual ConnParameter **getAllOutputParameters(char *scriptName) {return NULL;};
		/**
		 * Get all parmeters with direction out or inout from a given script.
		 * Return value:
		 *   NULL if the call went wrong else an array of ConnParameters ending with
		 *   a NULL index.
		 */
		virtual int doScriptCommand(char *command) {return CI_ERR_NOT_IMPLEMENTED;};
		/**
		 */
		virtual void parseScriptFile(char *filename) {return;};
		/**
		 */
		virtual int isConnected() {return 0;};
		/**
		 * Indicate if the inteface is connected
		 */
};//ConnectionInterface

#endif
