/***************************************************************************
                          ConnectionWrapper.h  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _CONNECTIONWRAPPER_H_
#define _CONNECTIONWRAPPER_H_

#include <time.h>
#include "ConnectionInterface.h"

#if defined(USE_PERCOMM)
#include "PerCommConnection.h"
#elif defined(USE_X3270)
#include "X3270Connection.h"
#elif defined(USE_EXEMUL)
#include "ExEmulConnection.h"
#endif

#ifdef WIN32
#include "ThreadIntefaceWin32.h"
#elif defined(POSIX_THREAD)
#include "ThreadIntefacePthread.h"
#endif

#ifndef BOOL
#define BOOL int
#endif
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FLASE
#define FLASE 0
#endif

// Defines for use with the Wrapper.doCommand() method.
#define CONN_COMM_NO_COMMAND 0
#define CONN_COMM_OPEN_CONNECTION 1
#define CONN_COMM_CLOSE_CONNECTION 2
#define CONN_COMM_DO_SCRIPT_COMMAND 3
#define CONN_COMM_DISCONNECT 4
#define CONN_COMM_FREE_CHAR_POINTER 5
#define CONN_COMM_MEMORY_ERROR 6


class ConnectionWrapper
{
	private:
		ConnectionId m_connId;
		ConnectionInterface *m_connIntf;
		Event m_setEvent, m_respEvent, m_discEvent;
		int m_connCommand;
		void *m_connParams;
		void *m_connResp;
		BOOL m_connThreadRunning, m_disconnecting;
		ThreadClass *m_connThread;

		void doCommandNoThread(int, void *);

	public:
		ConnectionWrapper();
		ConnectionWrapper(ConnectionId *, int connThreadOn=0);
		~ConnectionWrapper();
		void killConnectionWrapper();
		void init(ConnectionId *);

		int openConnection();
		int closeConnection();
		int setParameter(ConnParameter *);
		int getParameter(ConnParameter *);
		ConnParameter **getAllOutputParameters(char *);
		int doScriptCommand(char *);
		void parseScriptFile(char *);
		BOOL isConnected();

		BOOL isConnThreadRunning() {return m_connThreadRunning;};
		void *doCommand(int , void *);
		void doDisconnect();
		int getConnCommand() {return m_connCommand;};
		void *getConnParams() {return m_connParams;};
		void *getConnResp() {return m_connResp;};
		void setConnResp(void *resp) {m_connResp = resp;};
		int setEventWait() {return m_setEvent.wait();};
#ifdef POSIX_THREAD
		int setEventLockMutex() {return m_setEvent.lockEventMutext();};
#endif
		int respEventSet() {return m_respEvent.set();};
		int respDiscWait() {return m_discEvent.wait();};
		int respDiscSet() {return m_discEvent.set();};
#if defined (USE_PERCOMM)
		void setDisconnectCloseTerminalWindow() {if (m_connIntf) ((PerCommConnection *)m_connIntf)->setDisconnectCloseTerminalWindow();};
#elif defined (USE_X3270)
		void setDisconnectCloseTerminalWindow() {if (m_connIntf) ((X3270Connection *)m_connIntf)->setDisconnectCloseTerminalWindow();};
#elif defined (USE_EXEMUL)
		void setDisconnectCloseTerminalWindow() {if (m_connIntf) ((ExEmulConnection *)m_connIntf)->setDisconnectCloseTerminalWindow();};
#endif
};//ConnectionWrapper

#endif
