/***************************************************************************
                          ScreenConnection.cpp  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/

#include "ScreenConnection.h"

ScreenConnection::ScreenConnection()
{
	m_touchTime = time(NULL);
	m_conn = NULL;
	memset(&m_connId, 0 ,sizeof(ConnectionId));
	*m_symbolicConnName = '\0';
	m_inUse = FALSE;
	m_readyForDelete = FALSE;
	m_inErrorState = FALSE;
	m_pool = NULL;

	m_next = NULL;
}//ScreenConnection


ScreenConnection::~ScreenConnection()
{
	m_touchTime = time(NULL);
	closeConnection();
	if (m_conn) delete m_conn;
	m_conn = NULL;
}//~ScreenConnection


void ScreenConnection::killScreenConnection()
{
	if (m_conn) m_conn->killConnectionWrapper();
}//killScreenConnection


void ScreenConnection::initConnection(ConnectionId *connId)
{
	m_touchTime = time(NULL);
	if (m_conn) delete m_conn;
	memcpy(&m_connId, connId, sizeof(ConnectionId));
#if defined(USE_CONNTHREAD)
printf("initConnection using Connection thread\n");
	if ((m_conn = new ConnectionWrapper(&m_connId,1))==NULL) return;
#else
printf("initConnection does NOT use Connection thread\n");
	if ((m_conn = new ConnectionWrapper(&m_connId))==NULL) return;
#endif
	m_conn->parseScriptFile(connId->scriptsFileName);
}//initConnection


int ScreenConnection::openConnection()
{
	int rVal;

	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.openConnection()",m_symbolicConnName);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return ERR_NULL_WRAPPER;
	}
	if ((rVal=(int)m_conn->doCommand(CONN_COMM_OPEN_CONNECTION,NULL))!=CI_NO_ERR) {
		sprintf(logStr,"%s in openConnection() the doCommand(OPEN_CONNECTION) return with error: %d",m_symbolicConnName, rVal);
		m_pool->logWrite(1,logStr);
		m_readyForDelete = TRUE;
	}
	return rVal;
}//openConnection


int ScreenConnection::closeConnection()
{
	int rVal;

	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.closeConnection()",m_symbolicConnName);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return ERR_NULL_WRAPPER;
	}
	if ((rVal=(int)m_conn->doCommand(CONN_COMM_CLOSE_CONNECTION,NULL))!=CI_NO_ERR) {
		sprintf(logStr,"%s in closeConnection() the doCommand(CLOSE_CONNECTION) return with error: %d",m_symbolicConnName, rVal);
		m_pool->logWrite(1,logStr);
		m_readyForDelete = TRUE;
	}
	return rVal;
}//closeConnection



int ScreenConnection::setParameter(ConnParameter *param)
{
	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.setParameter(%s, %s, %s)",m_symbolicConnName, param->scriptName, param->paramName, param->value);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return ERR_NULL_WRAPPER;
	}
	return m_conn->setParameter(param);
}//setParameter


int ScreenConnection::getParameter(ConnParameter *param)
{
	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.getParameter(%s, %s)",m_symbolicConnName, param->scriptName, param->paramName);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return 0;
	}
	return m_conn->getParameter(param);
}//getParameter


ConnParameter **ScreenConnection::getAllOutputParameters(char *scriptName)
{
	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.getAllOutputParameters(%s)",m_symbolicConnName, scriptName);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return NULL;
	}
	return m_conn->getAllOutputParameters(scriptName);
}//getParameter


int ScreenConnection::doCommand(char *command)
{
	int rVal;

	m_touchTime = time(NULL);
	char logStr[256];
	sprintf(logStr,"%s.doCommand(%s)",m_symbolicConnName, command);
	m_pool->logWrite(3,logStr);
	if (m_conn==NULL) {
		m_readyForDelete = TRUE;
		return ERR_NULL_WRAPPER;
	}
	if ((rVal=(int)m_conn->doCommand(CONN_COMM_DO_SCRIPT_COMMAND,(void *)command))!=CI_NO_ERR) {
		sprintf(logStr,"%s.doCommand() the doCommand(%s) return with error: %d",command,m_symbolicConnName, rVal);
		m_pool->logWrite(1,logStr);
		m_readyForDelete = TRUE;
	}
	return rVal;
}//doCommand


BOOL ScreenConnection::isConnected()
{
	if (m_conn) return (BOOL)m_conn->isConnected();
	else return FALSE;
}//isConnected

void ScreenConnection::setSymbolicConnName(char *symbolicName)
{
	m_touchTime = time(NULL);
	if (symbolicName==NULL) return;
	strncpy(m_symbolicConnName, symbolicName, 64);
	m_symbolicConnName[64] = '\0';
}//setSymbolicConnName








ScreenConnectionPool::ScreenConnectionPool()
{
	strcpy(m_poolName,"ScreenConnectionPool");
	m_connTimeOutTime = 120;
	m_connNotUsedTime = 300;
	m_firstConn = m_lastConn = NULL;
	m_log = NULL;
	m_poolGoingDown = FALSE;
	m_poolIsReady = FALSE;
	*m_user = '\0';
	*m_password = '\0';
}//ScreenConnectionPool


ScreenConnectionPool::~ScreenConnectionPool()
{
	ScreenConnection *actConn, *delConn;

	actConn = m_firstConn;
	m_firstConn = NULL; // The pool do not return any connections
	while (actConn) {
		delConn = actConn;
		actConn = actConn->getNext();
		delete delConn;
	}
//	m_log->logWrite(1, "A ScreenConnectionPool object was destroyed");
	if (m_log) delete m_log;
}//~ScreenConnectionPool


void ScreenConnectionPool::poolConfigure(char *configFile)
{
	Element *xmlTree, *subXmlTree, *subTree;
	char *objName, logStr[512];
	ScreenConnection* conn;
	ConnectionId connId;
	long poolTimeOut;
	char *cp;

	strncpy(m_configFileName, configFile, _MAX_PATH);
	m_configFileName[_MAX_PATH] = '\0';
	strcpy(m_poolName, "ScreenConnectionPool");

	// Read and parse the config file
	m_miniXML.DropTree();
	m_miniXML.ParseFile(m_configFileName);
	xmlTree = m_miniXML.GetTree();
	// Search for log and object time out parameters
	xmlTree->InitSearch();
	if ((subTree=xmlTree->SearchFor("log-file-name"))!=NULL) {
		if ((cp=subTree->GetValue())!=NULL) {
			if (m_log) {
				m_log->close();
				char *cpo = m_log->getLogName();
				remove(cpo);
				delete m_log;
			}
			m_log = new Log(cp);
		}
	}
	if ((subTree=xmlTree->SearchFor("log-level"))!=NULL) {
		if ((cp=subTree->GetValue())!=NULL) {
			int level = atol(cp);
			if (level > 0) m_log->setLogLevel(level);
		}
	}

	if ((subTree=xmlTree->SearchFor("object-timeout-sec"))!=NULL) {
		if((cp=subTree->GetValue())!=NULL) poolTimeOut = atol(cp);
	}
	else poolTimeOut = 0;
	if (poolTimeOut > 0) setConnTimeout(poolTimeOut);

	if ((subTree=xmlTree->SearchFor("resource-timeout-sec"))!=NULL) {
		if((cp=subTree->GetValue())!=NULL) poolTimeOut = atol(cp);
	}
	else poolTimeOut = 0;
	if (poolTimeOut > 0) setConnNotUsedTime(poolTimeOut);

	// Search in the config file for paramters for the TransactionObj objects.
	xmlTree->InitSearch();
	subXmlTree = xmlTree->SearchFor("screen-connection-object");
	while (subXmlTree) {
		if ((subTree=subXmlTree->SearchFor("name"))!=NULL) objName = subTree->GetValue();
		else {
		}
		if ((subTree=subXmlTree->SearchFor("connection-id"))!=NULL) {
			char *attr;

#if defined (USE_PERCOMM)
			if ((attr=subTree->GetAttrubuteValue("session-name"))==NULL) connId.sessionName = '*';
			else connId.sessionName = *attr;
			if ((attr=subTree->GetAttrubuteValue("ws-profile"))==NULL) connId.wsProfileFileName[0] = '\0';
			else strcpy(connId.wsProfileFileName, attr);
			if ((attr=subTree->GetAttrubuteValue("scripts-file"))==NULL) *connId.scriptsFileName = '\0';
			else strcpy(connId.scriptsFileName, attr);
//			if ((attr=subTree->GetAttrubuteValue("emulator-command"))==NULL) *connId.emulCommand = '\0';
//			else strcpy((char *)connId.emulCommand, attr);
#elif defined(USE_EXEMUL)
			if ((attr=subTree->GetAttrubuteValue("session-name"))==NULL) connId.sessionName = '*';
			else connId.sessionName = *attr;
			if ((attr=subTree->GetAttrubuteValue("emulator-command"))==NULL) connId.emulCommand[0] = '\0';
			else strcpy(connId.emulCommand, attr);
			if ((attr=subTree->GetAttrubuteValue("scripts-file"))==NULL) *connId.scriptsFileName = '\0';
			else strcpy(connId.scriptsFileName, attr);
			if ((attr=subTree->GetAttrubuteValue("emulator-command"))==NULL) *connId.emulCommand = '\0';
			else strcpy((char *)connId.emulCommand, attr);
#elif defined(USE_X3270)
			if ((attr=subTree->GetAttrubuteValue("host"))==NULL) strcpy(connId.hostName,"localhost");
			else strcpy(connId.hostName, attr);
			if ((attr=subTree->GetAttrubuteValue("port"))==NULL) connId.port = 23;
			else connId.port = (short)atol(attr);
			if ((attr=subTree->GetAttrubuteValue("emulator-command"))==NULL) *connId.emulCommand = '\0';
			else strcpy((char *)connId.emulCommand, attr);
			if ((attr=subTree->GetAttrubuteValue("fifo-file"))==NULL) *connId.fifoFileName = '\0';
			else strcpy((char *)connId.fifoFileName, attr);
			if ((attr=subTree->GetAttrubuteValue("scripts-file"))==NULL) *connId.scriptsFileName = '\0';
			else strcpy(connId.scriptsFileName, attr);
#endif
			if ((attr=subTree->GetAttrubuteValue("time-out"))==NULL) connId.timeOutSec = 60;
			else connId.timeOutSec = atol(attr);
			strncpy(connId.symbolicConnName, objName, 65); connId.symbolicConnName[64] = '\0';
		}

		if ((conn = new ScreenConnection())==NULL) return;
		conn->initConnection(&connId);
		conn->setPool(this);
		conn->setSymbolicConnName(objName);
		insertConnection(conn);
		sprintf(logStr,"ScreenConnectionPool: atcivated Screen Connection Object %s",objName);
		logWrite(3,logStr);

		subXmlTree = xmlTree->SearchFor("screen-connection-object");
	}
	m_poolIsReady = TRUE;
}//poolConfigure


void ScreenConnectionPool::insertConnection(ScreenConnection *conn)
{
	char logStr[256];

	if (conn==NULL) return;
	sprintf(logStr,"ScreenConnectionPool.insertConnection(%s)",conn->getSymbolicConnName());
	logWrite(3,logStr);

	m_mutex.lock();
	if (m_firstConn==NULL) m_firstConn = m_lastConn = conn;
	else {
		m_lastConn->setNext(conn);
		m_lastConn = conn;
	}
	m_mutex.unlock();
}//insertConnection



ScreenConnection *ScreenConnectionPool::getFreeConnection()
{
	ScreenConnection *actConn;
	char logStr[512];

	m_mutex.lock();
	actConn = m_firstConn;
	while (actConn) {
		if (!actConn->isInUse() && !actConn->isReadyForDelete()) break;
		actConn = actConn->getNext();
	}
	if (actConn) {
		actConn->touchObj();
		actConn->reserveConnection();
		sprintf(logStr,"ScreenConnectionPool.getFreeConnection() Success! connection=%",actConn->getSymbolicConnName());
		logWrite(2,logStr);
	}
	else {
		sprintf(logStr,"ScreenConnectionPool.getFreeConnection() Failed!");
		logWrite(2,logStr);
	}
	m_mutex.unlock();
	if (actConn==NULL) return NULL;
	return actConn;
}//getFreeConnection


ScreenConnection *ScreenConnectionPool::getConnection(char *name)
{
	ScreenConnection *actConn;
	char logStr[512];

	m_mutex.lock();
	actConn = m_firstConn;
	while (actConn) {
//		if (strcmp(actConn->getSymbolicConnName(),name)==0) break;
		if ((strcmp(actConn->getSymbolicConnName(),name)==0) && !actConn->isInUse() && !actConn->isReadyForDelete()) break;
		actConn = actConn->getNext();
	}
	if (actConn) {
		actConn->touchObj();
		actConn->reserveConnection();
		sprintf(logStr,"ScreenConnectionPool.getConnection(%s) Success!",name);
		logWrite(2,logStr);
	}
	else {
		sprintf(logStr,"ScreenConnectionPool.getConnection(%s) Failed!",name);
		logWrite(2,logStr);
	}
	m_mutex.unlock();
	return actConn;
}//getConnection


ScreenConnection *ScreenConnectionPool::findConnection(char *name)
{
	ScreenConnection *actConn;
	char logStr[512];

	if (name==NULL) return NULL;

	m_mutex.lock();
	actConn = m_firstConn;
	while (actConn) {
		if ((strcmp(actConn->getSymbolicConnName(),name)==0) && !actConn->isReadyForDelete()) break;
		actConn = actConn->getNext();
	}
	if (actConn) {
		actConn->touchObj();
		sprintf(logStr,"ScreenConnectionPool.findConnection(%s) Success!",name);
		logWrite(2,logStr);
	}
	else {
		sprintf(logStr,"ScreenConnectionPool.findConnection(%s) Failed!",name);
		logWrite(2,logStr);
	}
	m_mutex.unlock();
	return actConn;
}//findConnection


void ScreenConnectionPool::unqueueConnection(ScreenConnection *conn)
{
	ScreenConnection *actConn;

	m_mutex.lock();
	if (conn==m_firstConn) {
		m_firstConn = m_firstConn->getNext();
		conn->setNext(NULL);
		if (m_firstConn==NULL) m_lastConn = NULL;
	}
	else {
		actConn = m_firstConn;
		while (actConn) {
			if (actConn->getNext()==conn) {
				actConn->setNext(conn->getNext());
				conn->setNext(NULL);
				if (conn==m_lastConn) m_lastConn = actConn;
				break;
			}
			actConn = actConn->getNext();
		}
	}
	m_mutex.unlock();
}//unqueueConnection


void ScreenConnectionPool::collectBlokedConnections()
/*
 * This method will delete all object a client have requested with out use them 
 * in the given object timeout.
 */
{
	ScreenConnection *actConn, *nxtConn, *newConn, *delConn;
	ConnParameter userParam, passParam;
	time_t nowTime;
	char logStr[512];

	nowTime = time(NULL);
	nxtConn = m_firstConn;
	while (nxtConn) {
		actConn = nxtConn;
		nxtConn = nxtConn->getNext();
		newConn = NULL;
		if ((actConn->isInUse() && ((actConn->getTouchTime()+m_connTimeOutTime) < nowTime)) || actConn->isReadyForDelete()) {
			//The object is proberly blocked delete it
			sprintf(logStr,"ScreenConnectionPool.collectBlokedObjects() prepare object <%s> to be collected",actConn->getSymbolicConnName());
			logWrite(2,logStr);
			// un-queue the object from the pool
			unqueueConnection(actConn);

			// Create a new object to substitute the old one
			if ((newConn = new ScreenConnection())==NULL) {
				logWrite(1,"ScreenConnectionPool.collectBlokedObjects() ERROR: could not create a new ScreenConnection");
				continue;
			}
			newConn->setPool(this);
			newConn->initConnection(actConn->getConnectionId());
			newConn->setSymbolicConnName(actConn->getSymbolicConnName());
			userParam.setScriptName("connect");
			userParam.setParamName("user");
			userParam.setValue(m_user);
			passParam.setScriptName("connect");
			passParam.setParamName("password");
			passParam.setValue(m_password);
			newConn->setParameter(&userParam);
			newConn->setParameter(&passParam);

			actConn->closeConnection();
			delConn = actConn;
		}
		else if (!actConn->isInUse() && actConn->isConnected() && ((actConn->getTouchTime()+m_connNotUsedTime) < nowTime)) {
			//The object has not been used for a long time close the connection to save resources
			actConn->reserveConnection();
			actConn->touchObj();
			sprintf(logStr,"ScreenConnectionPool.collectBlokedObjects() prepare object <%s> to be disconnected",actConn->getSymbolicConnName());
			logWrite(2,logStr);
//printf("> %s\n",logStr);
			actConn->setDisconnectCloseTerminalWindow();
			actConn->closeConnection();
			actConn->realseConnection();
			delConn = NULL;
		}
		else delConn = NULL;
		try {
			if (delConn) {
				printf(logStr,"ScreenConnectionPool.collectBlokedObjects() going to delete object <%s>",delConn->getSymbolicConnName());
				logWrite(2,logStr);
				delConn->killScreenConnection();
				delete delConn;
				logWrite(2,"ScreenConnectionPool.collectBlokedObjects() deleted object OK!");
			}
		} catch (...) {
			logWrite(1,"EXCEPTION... in ScreenConnectionPool.collectBlokedObjects()");
		}
		if (newConn) insertConnection(newConn);
	}
}//collectBlokedConnections


void ScreenConnectionPool::setUserPassword(char *user, char *password)
{
	ScreenConnection *actConn;
	ConnParameter userParam, passParam;

	strncpy(m_user, user, 15); m_user[14] = '\0';
	strncpy(m_password, password, 15); m_password[14] = '\0';

	userParam.setScriptName("connect");
	userParam.setParamName("user");
	userParam.setValue(user);
	passParam.setScriptName("connect");
	passParam.setParamName("password");
	passParam.setValue(password);

	actConn = m_firstConn;
	while (actConn) {
		actConn->setParameter(&userParam);
		actConn->setParameter(&passParam);
		actConn = actConn->getNext();
	}
}//setUserPassword
