/***************************************************************************
                          ScreenConnection.h  -  description
                             -------------------
    begin                : Sun Apr 21 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifndef _SCREENCONNECTION_H_
#define _SCREENCONNECTION_H_

#define SCREENCONNECTION_VERSION "0.5"

#include <time.h>
#include "ConnectionWrapper.h"
#include "Log.h"

#ifndef BOOL
#define BOOL int
#endif
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FLASE
#define FLASE 0
#endif

#define ERR_NULL_WRAPPER 1000

class ScreenConnectionPool;

class ScreenConnection
{
	private:
		time_t m_touchTime;
		ConnectionWrapper *m_conn;
		ConnectionId m_connId;
		char m_symbolicConnName[65];
		BOOL m_inUse;
		BOOL m_readyForDelete;
		BOOL m_inErrorState;
		ScreenConnectionPool *m_pool;
		ScreenConnection *m_next;

	public:
		ScreenConnection();
		~ScreenConnection();
		void killScreenConnection();

		void initConnection(ConnectionId *connId);

		int openConnection();
		int closeConnection();
		int setParameter(ConnParameter *param);
		int getParameter(ConnParameter *param);
		ConnParameter **getAllOutputParameters(char *scriptName);
		int doCommand(char *command);
		BOOL isConnected();

		ConnectionId *getConnectionId() {return &m_connId;};
		void setSymbolicConnName(char *symbolicName);
		char *getSymbolicConnName() {return m_symbolicConnName;};
		void setNext(ScreenConnection *next) {m_next = next;};
		ScreenConnection *getNext() {return m_next;};
		BOOL isInUse() {return m_inUse;};
		BOOL isReadyForDelete() {return m_readyForDelete;};
		BOOL reserveConnection() {if (m_inUse) return FALSE; m_inUse = TRUE; return TRUE;};
		BOOL isInErrorState() {return m_inErrorState;};
		void realseConnection() {m_inUse = FALSE;};
		time_t getTouchTime() {return m_touchTime;};
		void setPool(ScreenConnectionPool *pool) {m_pool = pool;};
		void touchObj() {m_touchTime = time(NULL);};
		void setDisconnectCloseTerminalWindow() {if (m_conn) m_conn->setDisconnectCloseTerminalWindow();};
};//ScreenConnection


class ScreenConnectionPool
{
	private:
		char m_poolName[255];
		char m_configFileName[_MAX_PATH+1];
		MiniXML m_miniXML;
		Log *m_log;
		time_t m_connTimeOutTime;
		time_t m_connNotUsedTime;
		BOOL m_poolGoingDown;
		BOOL m_poolIsReady;
		MutexClass m_mutex;
		ScreenConnection *m_firstConn, *m_lastConn;
		char m_user[15];
		char m_password[15];

	public:
		ScreenConnectionPool();
		~ScreenConnectionPool();

		void poolConfigure(char *configFile);
		void setLog(Log *log) {if (m_log) delete m_log; m_log=log;};
		void logWrite(int level, const char *message) {if (m_log) m_log->logWrite(level, message);};

		char *getPoolName() {return m_poolName;};
		void setPoolName(char *name) {strncpy(m_poolName, name, 254); m_poolName[254] = '\0';};
		void insertConnection(ScreenConnection *conn);
		void setConnTimeout(time_t seconds) {m_connTimeOutTime = seconds;};
		void setConnNotUsedTime(time_t seconds) {m_connNotUsedTime = seconds;};
		ScreenConnection *getFreeConnection();
		ScreenConnection *getConnection(char *name);
		ScreenConnection *findConnection(char *name);
		void unqueueConnection(ScreenConnection *conn);
		void collectBlokedConnections();
		void setPoolGoingDown(BOOL b) {m_poolGoingDown = b;};
		BOOL isGoingDown() {return m_poolGoingDown;};
		void setPoolIsReady(BOOL b) {m_poolIsReady = b;};
		BOOL isPoolReady() {return m_poolIsReady;};
		void setUserPassword(char *user, char *password);
};//ScreenConnectionPool


#endif
