/***************************************************************************
                          X3270Connection.cpp  -  description
                             -------------------
    begin                : Tue May 14 2002
    copyright            : (C) 2002 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef USE_X3270

#include "X3270Connection.h"
#include "ThreadIntefaceWin32.h"
#include "ThreadIntefacePthread.h"
#include <stdlib.h>

X3270Connection::X3270Connection()
{
	m_connectScriptDone = 0;
	m_closeTerminalWindow = 0;
	m_haveEmulatorOpen = 0;
	*m_symbolicConnName = '\0';
	m_connected = 0;
	m_scriptfileParsed = 0;
	m_timeOutSec = 0x7FFFF;
	m_x3270fp = NULL;
	m_x3270resp = -1;
}//X3270Connection


X3270Connection::~X3270Connection()
{
	if (m_x3270fp) pclose(m_x3270fp);
}//~X3270Connection


int X3270Connection::openConnection(ConnectionId *id)
{
	char s[1];
	long l, r;

	if (id==NULL) return CI_ERR_NULL_CONN_ID;

	strncpy(m_symbolicConnName, id->symbolicConnName, 65);
	m_symbolicConnName[64] = '\0';

	if (m_x3270fp==NULL) {
		if ((r=openTerminalWindow(id))!=X3270_RC_SUCCESS) return r;
	}

	m_connected = 1;

	if (*id->scriptsFileName=='\0') return CI_NO_ERR;
	if (!m_scriptfileParsed) {
		m_miniXML.DropTree();
		m_miniXML.ParseFile(id->scriptsFileName);
	}

//	if ((r=firstSteps())!=X3270_RC_SUCCESS) return CI_ERR_CONNECT;
	if ((r=firstSteps())!=X3270_RC_SUCCESS) return r;
	return CI_NO_ERR;
}//openConnection


int X3270Connection::closeConnection(ConnectionId *id)
{
	char s[1];
	long l, r;

	if (id==NULL) return CI_ERR_NULL_CONN_ID;

	if (!m_connected) {
		if (m_haveEmulatorOpen && m_closeTerminalWindow) closeTerminalWindow(id);
		return CI_NO_ERR;
	}

	r = lastSteps();
	if (m_closeTerminalWindow) {
		closeTerminalWindow(id);
	}
	m_connected = 0;
	return CI_NO_ERR;
}//closeConnection


int X3270Connection::setParameter(ConnParameter *param)
{
	Element *xmlTree, *subTree, *varTag;
	char *attrVal;

	if (param==NULL) return X3270_RC_BAD_PARAM;
	if (param->scriptName==NULL || param->paramName==NULL || param->value==NULL) return X3270_RC_BAD_PARAM;
	
	if ((xmlTree=m_miniXML.GetTree())==NULL) return X3270_RC_BAD_PARAM;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return X3270_RC_BAD_PARAM;

	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return X3270_RC_BAD_PARAM;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL) {
		if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->paramName)==0) {
			// Validate the direction
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"out")==0) return X3270_RC_BAD_PARAM;
			varTag->SetValue(param->value);
			return CI_NO_ERR;
		}
		varTag = varTag->GetNextElement();
	}

	return X3270_RC_BAD_PARAM;
}//setParameter


int X3270Connection::getParameter(ConnParameter *param)
{
	Element *xmlTree, *subTree, *varTag;
	char *attrVal;

	if (param==NULL) return X3270_RC_BAD_PARAM;
	if (param->scriptName==NULL || param->paramName==NULL) return X3270_RC_BAD_PARAM;

	if ((xmlTree=m_miniXML.GetTree())==NULL)  return X3270_RC_BAD_PARAM;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return X3270_RC_BAD_PARAM;

	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return X3270_RC_BAD_PARAM;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL ) {
		if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, param->paramName)==0) {
			// Validate the direction
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"in")==0) return X3270_RC_BAD_PARAM;
			if (param->value) free(param->value);
			param->setValue(varTag->GetValue());
			return CI_NO_ERR;
		}
		varTag = varTag->GetNextElement();
	}

	return X3270_RC_BAD_PARAM;
}//getParameter


ConnParameter **X3270Connection::getAllOutputParameters(char *scriptName)
{
	Element *xmlTree, *subTree, *varTag;
	char *varName, *attrVal;
	ConnParameter **params, *param;
	int maxParams = 10;
	int paramInx, i;

	if (scriptName==NULL) return NULL;


	if ((xmlTree=m_miniXML.GetTree())==NULL)  return NULL;
	xmlTree->InitSearch();
	subTree=xmlTree->SearchFor("script");
	while (subTree!=NULL) {
		if ((attrVal=subTree->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, scriptName)==0) {
			break;
		}
		subTree = xmlTree->SearchFor("script");
	}
	if (subTree==NULL) return NULL;

	if ((params = new ConnParameter *[maxParams])==NULL) return NULL;
	paramInx = 0;
	subTree->InitSearch();
	if ((subTree=subTree->SearchFor("params"))==NULL) return NULL;
	varTag = subTree->GetFirstSubElement();
	while (varTag!=NULL ) {
		if ((varName=varTag->GetAttrubuteValue("name"))!=NULL) {
			if ((attrVal=varTag->GetAttrubuteValue("direction"))==NULL || strcmp(attrVal,"in")==0) continue;
			if ((param = new ConnParameter())==NULL) continue;
			param->setScriptName(scriptName);
			param->setParamName(varName);
			param->setValue(varTag->GetValue());
			params[paramInx] = param;
			paramInx++;
			if (paramInx == (maxParams-1)) {
				ConnParameter **newParams;
				if ((newParams = new ConnParameter *[maxParams+10])==NULL) continue;
				for (i=0; i<maxParams; i++) {
					newParams[i] = params[i];
				}
				delete []params;
				params = newParams;
				maxParams += 10;
			}
		}
		varTag = varTag->GetNextElement();
	}
	params[paramInx] = NULL;

	
	return params;
}//getAllOutputParameters


int X3270Connection::parseCommands(Element *dataTree, Element *commTree)
{
	Element  *command, *subTag, *varTag;
	long p, l, r;
	char *tagName, *tagVal, *varName, ps[1921], *psPtr, *attrVal;

	if (commTree==NULL) return X3270_RC_BAD_PARAM;

	command = commTree->GetFirstSubElement();
	while (command!=NULL) {
		if ((tagName=command->GetName())==NULL) {
			command = commTree->GetNextSubElement();
			continue;
		}
		tagVal = command->GetValue();

		if (strcmp(tagName,"send-str")==0) {
			// do we have to send a variable or fixed string?
			if ((subTag=command->GetFirstSubElement())!=NULL) {
				if (strcmp(subTag->GetName(),"param-ref")==0) { //Send a variable
					if (dataTree==NULL) continue;
					// Find the name of the variable
					varName=subTag->GetAttrubuteValue("name");
					if (varName==NULL) continue;
					// Find the variable
					varTag = dataTree->GetFirstSubElement();
					while (varTag!=NULL) {
						if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, varName)==0) {
							tagVal = varTag->GetValue();
							if (tagVal!=NULL) r = sendString(tagVal);
							break;
						}
						varTag = varTag->GetNextElement();
					}
				}
			}
			else {//Send fixed string
				if (tagVal!=NULL) r = sendString(tagVal);
			}
		}
		else if (strcmp(tagName,"get-str")==0) {
			if ((r=copyPs2Str(ps, 1920))==X3270_RC_BAD_PARAM) {
				p = l = 0;
				if ((attrVal=command->GetAttrubuteValue("pos"))!=NULL) {
					p = atol(attrVal);
					if (p > 1920) p = 1920;
				}
				if ((attrVal=command->GetAttrubuteValue("len"))!=NULL) {
					l = atol(attrVal);
					if ((p+l) > 1920) l = 1920-p;
				}
				psPtr = &ps[p-1];
				psPtr[l] = '\0';
				if ((subTag=command->GetFirstSubElement())!=NULL) {
					if (strcmp(subTag->GetName(),"param-ref")==0) { //get the variable name
						if (dataTree==NULL) continue;
						// Find the name of the variable
						varName=subTag->GetAttrubuteValue("name");
						if (varName==NULL) continue;
						// Find the variable
						varTag = dataTree->GetFirstSubElement();
						while (varTag!=NULL) {
							if ((attrVal=varTag->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal, varName)==0) {
								varTag->SetValue(psPtr);
								break;
							}
							varTag = varTag->GetNextElement();
						}
					}
				}
			}
		}
		else if (strcmp(tagName,"set-cursor")==0) {
			l = atol(tagVal);
			r = setCursor(l);
		}
		else if (strcmp(tagName,"send-clear-key")==0) {
			r = sendClearKey();
		}
		else if (strcmp(tagName,"send-reset-key")==0) {
			r = sendResetKey();
		}
		else if (strcmp(tagName,"send-enter-key")==0) {
			r = sendEnterKey();
		}
		else if (strcmp(tagName,"send-pa-key")==0) {
			l = atol(tagVal);
			r = sendPaKey(l);
		}
		else if (strcmp(tagName,"send-tab-key")==0) {
			r = sendTabKey();
		}
		else if (strcmp(tagName,"send-backtab-key")==0) {
			r = sendBacktabKey();
		}
		else if (strcmp(tagName,"send-pf-key")==0) {
			l = atol(tagVal);
			r = sendPfKey(l);
		}
		else if (strcmp(tagName,"send-home-key")==0) {
			r = sendHomeKey();
		}
		else if (strcmp(tagName,"send-cursor-left-key")==0) {
			r = sendCursorLeftKey();
		}
		else if (strcmp(tagName,"send-cursor-right-key")==0) {
			r = sendCursorRightKey();
		}
		else if (strcmp(tagName,"send-cursor-up-key")==0) {
			r = sendCursorUpKey();
		}
		else if (strcmp(tagName,"send-cursor-down-key")==0) {
			r = sendCursorDownKey();
		}
		else if (strcmp(tagName,"wait")==0) {
			l = atol(tagVal);
			ThreadSleep(l);
		}
		if (r!=X3270_RC_SUCCESS) return r;

		command = commTree->GetNextSubElement();
	}

	return X3270_RC_SUCCESS;
}//parseCommands


int X3270Connection::parse(char *scriptName)
{
	Element *xmlTree, *dataTree, *firstStep, *subTree, *step, *commands, *response;
	long r, stop;
	char *cp, *nextStepName, ps[1921], *attrVal;

printf("X3270Connection.parse(): for %s\n",m_symbolicConnName);
	if ((xmlTree=m_miniXML.GetTree())==NULL)  return X3270_RC_BAD_PARAM;
	xmlTree->InitSearch();
	if ((subTree=xmlTree->SearchFor("scripts"))==NULL) return X3270_RC_BAD_PARAM;
	subTree->InitSearch();
	step = subTree->SearchFor("script");
	while (step!=NULL) {
		if ((attrVal=step->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal,scriptName)==0) {
			break;
		}
		step = subTree->SearchFor("script");
	}
	if (step==NULL) return X3270_RC_BAD_PARAM;
	subTree = step;

	subTree->InitSearch();
	dataTree = subTree->SearchFor("params");
	if ((firstStep=subTree->SearchFor("first-step"))==NULL) return X3270_RC_BAD_PARAM;
	if ((nextStepName=firstStep->GetValue())==NULL) return X3270_RC_BAD_PARAM;
	stop = 0;
	while (!stop) {
		if (nextStepName==NULL) return X3270_RC_SYSTEM_ERROR;
		if (strcmp(nextStepName,"stop")==0) {
			stop = 1;
			break;
		}
		if (strcmp(nextStepName,"error")==0) {
			stop = 1;
			return X3270_RC_SYSTEM_ERROR;
		}
		//Find the step
		subTree->InitSearch();
		do  {
			if ((step=subTree->SearchFor("step"))!=NULL) {
				if ((attrVal=step->GetAttrubuteValue("name"))!=NULL && strcmp(attrVal,nextStepName)==0) {
					break;
				}
			}
		} while (step!=NULL);
		if (step==NULL) return X3270_RC_BAD_PARAM;

		//Do the commands
		step->InitSearch();
		commands=step->SearchFor("commands");
		if ((r=parseCommands(dataTree, commands))!=X3270_RC_SUCCESS) return r;

		//Find next step
		if ((r=copyPs2Str(ps,1920))!=X3270_RC_SUCCESS) return r;
		ps[1920] = '\0';
		step->InitSearch();
		if ((firstStep=step->SearchFor("next"))!=NULL) nextStepName = nextStepName=firstStep->GetValue();
		else nextStepName = NULL;
		step->InitSearch();
		response=step->SearchFor("response");
		while (response!=NULL) {
			if ((cp=response->GetValue())!=NULL) {
				if (strstr(ps,cp) != NULL) {
					if ((attrVal=response->GetAttrubuteValue("next"))!=NULL) {
						nextStepName = attrVal;
						break;
					}
				}
			}
			response=step->SearchFor("response");
		}
	}

	return X3270_RC_SUCCESS;
}//parse


int X3270Connection::firstSteps()
{
	long r;
printf("X3270Connection.firstSteps(): for %s\n",m_symbolicConnName);
	if (!m_connectScriptDone) {
		if ((r=parse("connect"))!=X3270_RC_SUCCESS) return r;
		m_connectScriptDone = 1;
	}

	return X3270_RC_SUCCESS;
}//firstSteps


int X3270Connection::lastSteps()
{
	long r;

	r=parse("disconnect");
	m_connectScriptDone = 0;
	return r;
}//lastSteps


int X3270Connection::doScriptCommand(char *command)
{
	return parse(command);
}//doScriptCommand


void X3270Connection::parseScriptFile(char *filename)
{
	m_miniXML.DropTree();
	m_miniXML.ParseFile(filename);
	m_scriptfileParsed = 1;
}//parseScriptFile


int X3270Connection::getX3270Response()
{
	char *cp, str[128];
	int r;

#ifdef WIN32
#else
	if (m_x3270resp < 0) return X3770_RC_ERR_RESPONSE;
	if (m_x3270fp==NULL) return X3270_RC_INVALID_FP;
	memset(m_x3270buffer,0,3840);
	sprintf(str,"C(%s)",m_hostName);
	cp = m_x3270buffer;
	for (;;) {
		r = read(m_x3270resp, cp, 3840);
		if (r < 0) return X3770_RC_ERR_RESPONSE;
		if (r==0) break;
		if(strstr(m_x3270buffer, str)!=NULL) {
			break;
		}
		cp += r;
	}
//printf(m_x3270buffer);
#endif
	return X3270_RC_SUCCESS;
}//getX3270Response


int X3270Connection::x3270Wait(int timeout)
{
	int r;

	if (m_x3270fp==NULL) return X3270_RC_INVALID_FP;
printf("> x3270Wait(%d)\n",timeout);

	fprintf(m_x3270fp,"Wait(%d,InputField)\n",timeout);
	fflush(m_x3270fp);
	r = getX3270Response();
	return r;
}//x3270Wait


int X3270Connection::sendString(char *data)
{
	char s[2], *cp;;
	long r;

	if (m_x3270fp==NULL) return X3270_RC_INVALID_FP;

printf("> sendString(%s)\n",data);
//	fprintf(m_x3270fp,"Wait(%d,InputField)\nString(\"%s\")\n",m_timeOutSec,data);
	fprintf(m_x3270fp,"String(\"%s\")\n",data);
	fflush(m_x3270fp);
	r = getX3270Response();

	return r;
}//sendString


int X3270Connection::copyPs2Str(char *data, long maxLength)
{
	char *cp, *bp, *dp;
	int i,j ,inx, r;

printf("> copyPs2Str()\n");
	fprintf(m_x3270fp,"MoveCursor(0,0)\n");
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"Ascii(1920)\n");
	fflush(m_x3270fp);
	if ((r=getX3270Response()) != X3270_RC_SUCCESS) {
		return r;
	}
	dp = data;
	bp = m_x3270buffer;
	inx = 0;
	for (i=0; i<24; i++) {
		if ((cp=strstr(bp,"data: "))==NULL) {
			return X3270_RC_INVALID_PS;
		}
		cp += 6;
		bp += 86;
		for (j=0; j<80; j++) {
			*dp = *cp;
			dp++; cp++;
			inx++;
			if (inx >= maxLength) break;
		}
		if (inx >= maxLength) break;
	}
	fprintf(m_x3270fp,"Home()\n");
	fflush(m_x3270fp);
	if ((r=getX3270Response()) != X3270_RC_SUCCESS) {
		return r;
	}
	data[maxLength] = '\0';
//printf("Data>\n%s",data);
	
	return X3270_RC_SUCCESS;
}//copyPs2Str


int X3270Connection::copyStr2Ps(char *data, long length)
{
	char ps[1921];
	long l;

	if (length > 1920) l = 1920;
	else l = length;
	strncpy(ps, data, l);
	ps[l] = '\0';
printf("> copyStr2Ps(%s)\n",ps);
	fprintf(m_x3270fp,"MoveCursor(0,0)\n");
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"String(\"%s\")\n",ps);
	fflush(m_x3270fp);
	return getX3270Response();
}//copyStr2Ps


int X3270Connection::copyStr2Ps(char *data, long length, long pos)
/*
 * Upper-left cornor (row 1, column 1) is position 1, pos=[1..1920]
 */
{
	char ps[1921];
	long r, c, l;

	if (pos<1 || pos>1920) return X3270_RC_BAD_PARAM;

	if (length > 1920) l = 1920;
	else l = length;
	strncpy(ps, data, l);
	ps[l] = '\0';
	r = (pos-1)/80;
	c = (pos-1)%80;
printf("> copyStr2Ps(%s,%d)\n",ps,pos);
	fprintf(m_x3270fp,"MoveCursor(%d,%d)\n",r,c);
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"String(\"%s\")\n",ps);
	fflush(m_x3270fp);
	return getX3270Response();
}//copyStr2Ps


int X3270Connection::setCursor(long pos)
{
	long r, c, l;

	if (pos<1 || pos>1920) return X3270_RC_BAD_PARAM;
printf("> setCursor(%d)\n",pos);

	r = (pos-1)/80;
	c = (pos-1)%80;
	fprintf(m_x3270fp,"MoveCursor(%d,%d)\n",r,c);
	fflush(m_x3270fp);
	return getX3270Response();
}//setCursor


int X3270Connection::sendClearKey()
{
printf("> sendClearKey()\n");
	fprintf(m_x3270fp,"Clear()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendClearKey


int X3270Connection::sendResetKey()
{
printf("> sendResetKey()\n");
	fprintf(m_x3270fp,"Reset()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendResetKey


int X3270Connection::sendEnterKey()
{
printf("> sendEnterKey()\n");
	fprintf(m_x3270fp,"Enter()\n");
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"Wait(%d,Output)\n",m_timeOutSec);
	fflush(m_x3270fp);
	return getX3270Response();
}//sendEnterKey


int X3270Connection::sendPaKey(int key)
{
	if (key<1 || key>3) return X3270_RC_BAD_PARAM;
printf("> sendPaKey(%d)\n",key);

	fprintf(m_x3270fp,"PA(%d)\n",key);
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"Wait(%d,Output)\n",m_timeOutSec);
	fflush(m_x3270fp);
	return getX3270Response();
}//sendPa1Key


int X3270Connection::sendTabKey()
{
printf("> sendTabKey()\n");
	fprintf(m_x3270fp,"Tab()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendTabKey


int X3270Connection::sendBacktabKey()
{
printf("> sendBacktabKey()\n");
	fprintf(m_x3270fp,"BackTab()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendBacktabKey


int X3270Connection::sendPfKey(int key)
{
	if (key<1 || key>24) return X3270_RC_BAD_PARAM;
printf("> sendPfKey(%d)\n",key);

	fprintf(m_x3270fp,"PF(%d)\n",key);
	fflush(m_x3270fp);
	getX3270Response();
	fprintf(m_x3270fp,"Wait(%d,Output)\n",m_timeOutSec);
	fflush(m_x3270fp);
	return getX3270Response();
}//sendPfKey


int X3270Connection::sendHomeKey()
{
printf("> sendHomeKey()\n");
	fprintf(m_x3270fp,"Home()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendHomeKey


int X3270Connection::sendCursorLeftKey()
{
	fprintf(m_x3270fp,"Left()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendCursorLeftKey


int X3270Connection::sendCursorRightKey()
{
	fprintf(m_x3270fp,"Right()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendCursorRightKey


int X3270Connection::sendCursorUpKey()
{
	fprintf(m_x3270fp,"Up()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendCursorUpKey


int X3270Connection::sendCursorDownKey()
{
	fprintf(m_x3270fp,"Down()\n");
	fflush(m_x3270fp);
	return getX3270Response();
}//sendCursorDownKey


int X3270Connection::openTerminalWindow(ConnectionId *id)
{
	char command[2500];
	int r;

	if (m_x3270fp!=NULL && m_x3270resp>0) return X3270_RC_SUCCESS;

printf("X3270Connection.openTerminalWindow(): for %s\n",m_symbolicConnName);
	sprintf(command,"%s | pipeFifo %s",id->emulCommand,id->fifoFileName);
printf("> Command %s \n> Host: %s:%d\n",command,id->hostName, id->port);
	if (m_x3270fp!=NULL) pclose(m_x3270fp);
	if (m_x3270resp >= 0) close(m_x3270resp);
	if ((m_x3270fp=popen(command,"w"))==NULL) return X3270_RC_TERM_OPEN_ERR;
	ThreadSleep(id->timeOutSec);
	if ((m_x3270resp=open(id->fifoFileName, O_RDONLY))<0) {
		pclose(m_x3270fp);
		m_x3270fp = NULL;
		return X3270_RC_TERM_OPEN_ERR;
	}
	m_timeOutSec = id->timeOutSec;
	strcpy(m_hostName, id->hostName);
	fprintf(m_x3270fp,"Connect(%s:%d)\nWait(%d,InputField)\n",id->hostName, id->port, m_timeOutSec);
	fflush(m_x3270fp);
	if ((r=getX3270Response()) != X3270_RC_SUCCESS) return X3270_RC_TERM_NOT_CONN;

	return X3270_RC_SUCCESS;
}//openTerminalWindow


int X3270Connection::closeTerminalWindow(ConnectionId *id)
{
	if (m_x3270resp >= 0) close(m_x3270resp);
	m_x3270resp = -1;
	if (m_x3270fp!=NULL) pclose(m_x3270fp);
	m_x3270fp = NULL;
	m_connected = 0;

	return X3270_RC_SUCCESS;
}//closeTerminalWindow

#endif
