/***************************************************************************
                          WinMain.cpp  -  description
                             -------------------
    begin                : Fri Jan 3 2003
    copyright            : (C) 2003 by Henrik Baastrup
    email                : hba@libero.it
    home page            : http://digilander.iol.it/hba/Linux/index.html
 ***************************************************************************/

/***************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU GPL (General Public License) as published
 *   by the Free Software Foundation:
 *   http://www.gnu.org/copyleft/gpl.html or in the file copying.txt
 *
 ***************************************************************************/
#ifdef WIN32

#include "Version.h"
#include "Hermod.h"
#include <winbase.h>
#include "WinMainMsg.h"

char SERVICE_NAME[] = "Hermod";

extern int HermodBind(char *, char *, char *, short, char *);
extern void StopHermod(short);
int ReadRegistry();

char XmlConfigFileName[_MAX_PATH+1];
char HttpLogFileName[_MAX_PATH+1];
int UsePassedConfigFile = 0;
int UsePassedHttpPort = 0;
short HttpPort;
char MainframeUser[15];
char MainframePassword[15];
HANDLE hEventLogSource = NULL;
HANDLE mainServiceThreadHandle = 0;

#ifdef __cplusplus
extern "C" {
void PrintOut(FILE *fp, const char *str);
void LogEventEntry(WORD wType, DWORD dwId, const char *pszS1, const char *pszS2, const char *pszS3);
void StopService();
}
#endif


void PrintHelp()
{
	printf("\n");
	printf("Hermod syntax:\n");
	printf("\\nHermod [-g|-f file|-p port|-i|-u|-v|-h]\n");
	printf("\nExplanation:\n");
	printf(" -g: run Hermod interactive and not as a service\n");
	printf(" -f: file: use the file as configoration file.\n");
	printf(" -p: port: use the port as HTTP listen port\n");
	printf(" -i: install the application.\n");
	printf(" -u: un-install the application.\n");
	printf(" -v: print the version of the application.\n");
	printf(" -h: print this help text.\n");
}//PrintHelp


void LogEventEntry(WORD wType, DWORD dwId, const char *pszS1, const char *pszS2, const char *pszS3)
/*
 * Write in the Event Log
 */
{
	BOOL b;
	const char *ps[3];
	int iStr;
	
	ps[0] = pszS1;
	ps[1] = pszS2;
	ps[2] = pszS3;
	iStr = 0;
	for (int i=0; i<3; i++) if (ps[i]!=NULL) iStr++;

	if (!hEventLogSource) {
		hEventLogSource = RegisterEventSource(NULL,SERVICE_NAME);
	}

	if (hEventLogSource) {
		b = ReportEvent(hEventLogSource, wType,0,dwId,NULL,iStr,0,ps,NULL);
		if (!b) {
			DWORD dw;
			dw = GetLastError();
		}
	}
}//LogEventEntry


DWORD MainServiceThread(LPWORD param)
/*
 * This is the actual program/thread for the service
 */
{
	ReadRegistry();

	// Startup the Windows socket layer
	WORD wVersionRequested = MAKEWORD( 2, 2 );
	WSADATA wsaData;
	WSAStartup (wVersionRequested, &wsaData);

	HermodBind(XmlConfigFileName, MainframeUser, MainframePassword, HttpPort, HttpLogFileName);

	WSACleanup();

	ExitThread(0);

	//We never arrive here, but it makes the compiler happy:-)
	return 0;
}//MainServiceThread


/*********** Service Registration, Instaltion and Uninsatalation Stuff ***************/

#define DISPLAY_NAME "Hermod"

HANDLE terminateEvent = NULL;
SERVICE_STATUS_HANDLE serviceStatusHandle;
BOOL pauseService = FALSE;
BOOL runningService = FALSE;

int CreateRegistry()
{
    char szKey[256];
    HKEY hKey = NULL;
	DWORD dWord;

	// Add private keys for the server
	strcpy(szKey, "SOFTWARE\\HBaastrup\\Hermod\\");
    if (RegCreateKey(HKEY_LOCAL_MACHINE, szKey, &hKey) != ERROR_SUCCESS) {
		printf("Error in create key in registry: %s\n",szKey);
        return 1;
    }
	dWord = (DWORD) HttpPort;
	RegSetValueEx(hKey,
                    "HttpPort",
                    0,
                    REG_DWORD, 
                    (CONST BYTE*)&dWord,
                    sizeof(DWORD));
	// Add config file name
	RegSetValueEx(hKey,
                    "XmlConfigFile",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)XmlConfigFileName,
                    strlen(XmlConfigFileName) + 1);
	// Add http log file name
	RegSetValueEx(hKey,
                    "HttpLogFileName",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)HttpLogFileName,
                    strlen(HttpLogFileName) + 1);
	// Add Mainframe user id
	RegSetValueEx(hKey,
                    "MainframeUser",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)MainframeUser,
                    strlen(MainframeUser) + 1);
	// Add Mainframe password
	RegSetValueEx(hKey,
                    "MainframePassword",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)MainframePassword,
                    strlen(MainframePassword) + 1);
#ifdef USE_PERCOMM
	// Add HLLAPI DLL file name
	RegSetValueEx(hKey,
                    "HllapiFile",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)HllapiFileName,
                    strlen(HllapiFileName) + 1);
	// Add PCSAPI DLL file name
	RegSetValueEx(hKey,
                    "PcsapiFile",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)PcsapiFileName,
                    strlen(PcsapiFileName) + 1);
#elif defined (USE_EXEMUL)
	// Add HLLAPI DLL file name
	RegSetValueEx(hKey,
                    "HllapiFile",
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)HllapiFileName,
                    strlen(HllapiFileName) + 1);
#endif
    RegCloseKey(hKey);
	return 0;
}//CreateRegistry


int ReadRegistry()
{
    char szKey[256];
    HKEY hKey = NULL;
	DWORD regVal;
	DWORD rType;
	DWORD dWord;

	//Read registry for start-up values
	strcpy(szKey, "SOFTWARE\\HBaastrup\\Hermod\\");
    if (RegOpenKey(HKEY_LOCAL_MACHINE, szKey, &hKey) != ERROR_SUCCESS) {
        return 1;
    }
	if (!UsePassedConfigFile) {
		regVal = _MAX_PATH;
		RegQueryValueEx(hKey, "XmlConfigFile",NULL, &rType, (unsigned char *)XmlConfigFileName, &regVal);
	}
	regVal = _MAX_PATH;
	RegQueryValueEx(hKey, "HttpLogFileName",NULL, &rType, (unsigned char *)HttpLogFileName, &regVal);
	regVal = sizeof(DWORD);
	if (!UsePassedHttpPort) {
		if (RegQueryValueEx(hKey, "HttpPort",NULL, &rType, (unsigned char *)&dWord, &regVal)==ERROR_SUCCESS) HttpPort = (short)dWord;
		HttpPort = (short)dWord;
	}
	regVal = 15;
	RegQueryValueEx(hKey, "MainframeUser",NULL, &rType, (unsigned char *)MainframeUser, &regVal);
	regVal = 15;
	RegQueryValueEx(hKey, "MainframePassword",NULL, &rType, (unsigned char *)MainframePassword, &regVal);
#ifdef USE_PERCOMM
	regVal = _MAX_PATH;
	RegQueryValueEx(hKey, "HllapiFile",NULL, &rType, (unsigned char *)HllapiFileName, &regVal);
	regVal = _MAX_PATH;
	RegQueryValueEx(hKey, "PcsapiFile",NULL, &rType, (unsigned char *)PcsapiFileName, &regVal);
#elif defined (USE_EXEMUL)
	regVal = _MAX_PATH;
	RegQueryValueEx(hKey, "HllapiFile",NULL, &rType, (unsigned char *)HllapiFileName, &regVal);
#endif
    RegCloseKey(hKey);
	return 0;
}//ReadRegistry


void StopService()
{
	SC_HANDLE service, scm;
	BOOL success;
	SERVICE_STATUS status;
	char err[512];

	scm = OpenSCManager(0, 0, SC_MANAGER_CREATE_SERVICE);
	if (!scm) {
		sprintf(err,"Error in openSCManager. Error number: %d\n",GetLastError());
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		return;
	}
	service = OpenService(scm, SERVICE_NAME, SERVICE_ALL_ACCESS | WRITE_OWNER);
	if (!service) {
		sprintf(err,"Error in OpenService. Error number: %d\n",GetLastError());
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		return;
	}
	success = ControlService(service, SERVICE_CONTROL_STOP, &status);
	if (!success) {
		sprintf(err,"Error in ControlService. Error number: %d\n",GetLastError());
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		return;
	}
}//StopService


void UnInstallService()
/*
 * Uninstall the service from the Service manager.
 */
{
	SC_HANDLE service, scm;
	BOOL success;
	SERVICE_STATUS status;
	char err[512];
	char sysDir[513], servPath[550], st[512];
	BOOL b;

	scm = OpenSCManager(0, 0, SC_MANAGER_CREATE_SERVICE);
	if (!scm) {
		sprintf(err,"Error in openSCManager. Error number: %d\n",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	service = OpenService(scm, SERVICE_NAME, SERVICE_ALL_ACCESS | DELETE);
	if (!service) {
		sprintf(err,"Error in OpenService. Error number: %d\n",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	success = QueryServiceStatus(service, &status);
	if (!success) {
		sprintf(err,"Error in QueryServiceStatus. Error number: %d\n",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	if (status.dwCurrentState != SERVICE_STOPPED) {
		printf("Stopping service...\n");
		success = ControlService(service, SERVICE_CONTROL_STOP, &status);
		if (!success) {
			sprintf(err,"Error in ControlService. Error number: %d\n",GetLastError());
			printf("%s\n",err);
			LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
			exit(0);
		}
	}

	success = DeleteService(service);
	if (!success) {
		sprintf(err,"Error in DeleteService. Error number: %d\n",GetLastError());
		exit(0);
	}

	if (!GetSystemDirectory(sysDir,512)) {
		sprintf(err,"Error in GetSystemDirectory. Error number: %d\n",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_REMOVED_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	/* Do all the registery and file cleanup here! */
	sprintf(servPath,"%s\\Hermod.exe",sysDir);
	b = DeleteFile(st);

	printf("Service removed\n");
	CloseServiceHandle(service);
	CloseServiceHandle(scm);
	LogEventEntry(EVENTLOG_INFORMATION_TYPE, EVMSG_REMOVED, SERVICE_NAME, "Service removed!", NULL);
}//UnInstallService


void InstallService(char *serviceUserId, char *servicePassword)
/*
 * Install the service in the Service manager.
 */
{
	SC_HANDLE newService, scm;
	char sysDir[513], servPath[550];
	char err[512];
	BOOL b;

	if (!GetSystemDirectory(sysDir,512)) {
		sprintf(err,"Error in GetSystemDirectory. Error number: %d",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_INSTALL_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}
	sprintf(servPath,"%s\\Hermod.exe",sysDir);
	b = CopyFile("Hermod.exe", servPath, TRUE);

	CreateRegistry();

	scm = OpenSCManager(0, 0, SC_MANAGER_CREATE_SERVICE);
	if (!scm) {
		sprintf(err,"Error in openSCManager. Error number: %d",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_INSTALL_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	newService = CreateService(scm,
								SERVICE_NAME,
								DISPLAY_NAME,
								SERVICE_ALL_ACCESS,
								SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
								SERVICE_AUTO_START,
//								SERVICE_DEMAND_START,
								SERVICE_ERROR_NORMAL,
								servPath,
								0,0,0,serviceUserId,servicePassword);
	if (!newService) {
		sprintf(err,"Error in CreateService. Error number: %d",GetLastError());
		printf("%s\n",err);
		LogEventEntry(EVENTLOG_ERROR_TYPE, EVMSG_INSTALL_ERR, SERVICE_NAME, err, NULL);
		exit(0);
	}

	/* Do all settings of registry and copy of files here */

	printf("Service installed!\n");
	LogEventEntry(EVENTLOG_INFORMATION_TYPE, EVMSG_INSTALLED, SERVICE_NAME, "Hermod service installed!", NULL);
	CloseServiceHandle(newService);
	CloseServiceHandle(scm);
}//InstallService


BOOL SendStatusToSCM(DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwServiceSpecificExitCode, DWORD dwCheckPoint, DWORD dwWaitHint)
/*
 * Send status to the Service manager
 */
{
	BOOL success;
	SERVICE_STATUS serviceStatus;

	serviceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	serviceStatus.dwCurrentState = dwCurrentState;

	if (dwCurrentState==SERVICE_START_PENDING) serviceStatus.dwControlsAccepted = 0;
	else serviceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_PAUSE_CONTINUE | SERVICE_ACCEPT_SHUTDOWN;

	if (dwServiceSpecificExitCode==0) serviceStatus.dwWin32ExitCode = dwWin32ExitCode;
	else serviceStatus.dwWin32ExitCode = ERROR_SERVICE_SPECIFIC_ERROR;
	serviceStatus.dwServiceSpecificExitCode = dwServiceSpecificExitCode;

	serviceStatus.dwCheckPoint = dwCheckPoint;
	serviceStatus.dwWaitHint = dwWaitHint;

	success = SetServiceStatus(serviceStatusHandle, &serviceStatus);

	return success;
}//SendStatusToSCM


BOOL InitService()
/*
 * Initialize the actual servic program by starting service thread there do the networking
 */
{
	DWORD id;

	mainServiceThreadHandle = CreateThread(0, 0, (LPTHREAD_START_ROUTINE) MainServiceThread, 0, 0, &id);
	if (mainServiceThreadHandle==0) return FALSE;
	runningService = TRUE;
	LogEventEntry(EVENTLOG_INFORMATION_TYPE, EVMSG_STARTED, SERVICE_NAME, "Service was started!", NULL);
	return TRUE;
}//InitService


void ServiceHandle(DWORD controlCode)
/*
 * Handle event from the service
 */
{
	DWORD currentState = 0;
	BOOL success;

	switch (controlCode) {
		case SERVICE_CONTROL_STOP:
			success = SendStatusToSCM(SERVICE_STOP_PENDING, NO_ERROR, 0, 1, 5000);
			runningService = FALSE;
			SetEvent(terminateEvent);
			return;

		case SERVICE_CONTROL_PAUSE:
			if (runningService && !pauseService) {
				success = SendStatusToSCM(SERVICE_PAUSE_PENDING, NO_ERROR, 0, 1, 1000);
				pauseService = TRUE;
				SuspendThread(mainServiceThreadHandle);
//				SuspendHermod();
				currentState = SERVICE_PAUSED;
			}
			break;

		case SERVICE_CONTROL_CONTINUE:
			if (runningService && pauseService) {
				success = SendStatusToSCM(SERVICE_CONTINUE_PENDING, NO_ERROR, 0, 1, 1000);
				pauseService = FALSE;
				ResumeThread(mainServiceThreadHandle);
//				ResumeHermod();
				currentState = SERVICE_RUNNING;
			}
			break;

		case SERVICE_CONTROL_INTERROGATE:
			break;

		case SERVICE_CONTROL_SHUTDOWN:
			return;

		default:
			break;
	}

	SendStatusToSCM(currentState, NO_ERROR, 0, 0, 0);
}//ServiceHandle


void terminate(DWORD error)
/*
 * Terminat the service.
 */
{
	StopHermod(HttpPort);

	if (terminateEvent) CloseHandle(terminateEvent);

	if (serviceStatusHandle) SendStatusToSCM(SERVICE_STOPPED, error, 0, 0, 0);

	LogEventEntry(EVENTLOG_INFORMATION_TYPE, EVMSG_STOPED, SERVICE_NAME, "Service was stoped", NULL);
}//terminate


void ServiceMain(int argc, char *argv)
/*
 * Main function of the service. Is called from main when the service is started
 * but not when the service is called with an argument for install/unistall or help.
 */
{
	BOOL success;

	serviceStatusHandle = RegisterServiceCtrlHandler(SERVICE_NAME,(LPHANDLER_FUNCTION)ServiceHandle);
	if (!serviceStatusHandle) {terminate(GetLastError()); return;}

	success = SendStatusToSCM(SERVICE_START_PENDING, NO_ERROR, 0, 1, 5000);
	if (!success) {terminate(GetLastError()); return;}

	terminateEvent = CreateEvent(0, TRUE, FALSE, 0);
	if (!terminateEvent) {terminate(GetLastError()); return;}

	success = SendStatusToSCM(SERVICE_START_PENDING, NO_ERROR, 0, 2, 5000);
	if (!success) {terminate(GetLastError()); return;}

	success = InitService();
	if (!success) {terminate(GetLastError()); return;}

	success = SendStatusToSCM(SERVICE_RUNNING, NO_ERROR, 0, 0, 0);
	if (!success) {terminate(GetLastError()); return;}

	WaitForSingleObject(terminateEvent, INFINITE);

	terminate(0);
}//ServiceMain


int main(int argc, char *argv[])
{
	int runInteractive = 0;
	SERVICE_TABLE_ENTRY serviceTable[] = {
		{ SERVICE_NAME, (LPSERVICE_MAIN_FUNCTION) ServiceMain},
		{NULL, NULL}
	};
	BOOL success;

	// Initialize some globals variables
#ifdef USE_PERCOMM
	strcpy(HllapiFileName, "C:\\Program Files\\Personal Communications\\EHLAPI32.dll");
	strcpy(PcsapiFileName, "C:\\Program Files\\Personal Communications\\pcsapi32.dll");
#elif defined (USE_EXEMUL)
	strcpy(HllapiFileName, "C:\\Program Files\\Quick3270\\Qhllapi.dll");
#endif
	strcpy(XmlConfigFileName, "C:\\config.xml");
	strcpy(HttpLogFileName, "C:\\HermodHttp.log");
	HttpPort = 5080;
	strcpy(MainframeUser,"USER");
	strcpy(MainframePassword,"PASSWORD");


	// Interpret the program aruments
	for (int i=1; i<argc; i++) {
		if (strcmp(argv[i],"-h")==0) {
			PrintHelp();
			return 0;
		}
		else if (strcmp(argv[i],"-v")==0) {
			printf("\n%s\n",VERSION);
#if defined (USE_PERCOMM)
			printf("  for IBM's Personal Communications Emulator\n");
#elif defined(USE_X3270)
			printf("  for X-3270 Emulator\n");
#elif defined(USE_EXEMUL)
			printf("  for External Windows Emulator\n");			
#endif
			return 0;
		}
		else if (strcmp(argv[i],"-f")==0) {
			i++;
			if (i < argc) {
				strcpy(XmlConfigFileName, argv[i]);
				UsePassedConfigFile = 1;
			}
		}
		else if (strcmp(argv[i],"-p")==0) {
			i++;
			if (i < argc) {
				HttpPort = (short)atol(argv[i]);
				UsePassedHttpPort = 1;
			}
		}
		else if (strcmp(argv[i],"-g")==0) {
			runInteractive = 1;
		}
		else if (strcmp(argv[i],"-i")==0) {
			InstallService(NULL,NULL);
			return 0;
		}
		else if (strcmp(argv[i],"-u")==0) {
			UnInstallService();
			return 0;
		}
	}

	if (runInteractive) {
		printf("Hermod starting\n%s\n",VERSION);
		MainServiceThread(NULL);
	}

	success = StartServiceCtrlDispatcher(serviceTable);
	if (!success) {
		printf("Error in StartServiceCtrlDispatcher. Error number: %d\n",GetLastError());
	}

	return 1;
}//main

#endif
